@@ -34,7 +34,7 @@
 #include "rb-podcast-manager.h"
 #include "rb-file-helpers.h"
 #include "rb-debug.h"
-#include "rb-podcast-marshal.h"
+#include "rb-marshal.h"
 #include "rhythmdb.h"
 #include "rhythmdb-query-model.h"
 #include "rb-podcast-parse.h"
@@ -122,7 +122,7 @@ typedef struct
 typedef struct
 {
 	RBPodcastManager *pd;
-	char* url;
+	char *url;
 } RBPodcastThreadInfo;
 
 
@@ -143,8 +143,6 @@ static void rb_podcast_manager_get_property 		(GObject *object,
 		                                	 GValue *value,
                 		                	 GParamSpec *pspec);
 static void rb_podcast_manager_copy_post 		(RBPodcastManager *pd);
-static int rb_podcast_manager_mkdir_with_parents 	(const gchar *pathname,
-					  		 int mode);
 static gboolean rb_podcast_manager_sync_head_cb 	(gpointer data);
 static gboolean rb_podcast_manager_head_query_cb 	(GtkTreeModel *query_model,
 						   	 GtkTreePath *path, 
@@ -152,7 +150,7 @@ static gboolean rb_podcast_manager_head_query_cb 	(GtkTreeModel *query_model,
 						   	 RBPodcastManager *data);
 static gboolean rb_podcast_manager_save_metadata	(RhythmDB *db, 
 						  	 RhythmDBEntry *entry, 
-						  	 const char* uri);
+						  	 const char *uri);
 static void rb_podcast_manager_db_entry_added_cb 	(RBPodcastManager *pd, 
 							 RhythmDBEntry *entry);
 static void rb_podcast_manager_db_entry_deleted_cb 	(RBPodcastManager *pd, 
@@ -214,7 +212,7 @@ rb_podcast_manager_class_init (RBPodcastManagerClass *klass)
 		 		GTK_RUN_LAST,
 				G_STRUCT_OFFSET (RBPodcastManagerClass, status_changed),
 				NULL, NULL,
-				rb_podcast_marshal_VOID__POINTER_ULONG,
+				rb_marshal_VOID__POINTER_ULONG,
 				G_TYPE_NONE,
 				2,
 				G_TYPE_POINTER,
@@ -435,10 +433,12 @@ rb_podcast_manager_download_entry (RBPodcastManager *pd, RhythmDBEntry *entry)
 gboolean
 rb_podcast_manager_entry_downloaded (RhythmDBEntry *entry)
 {
-	g_assert (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
-
 	gulong status;
 	const gchar *file_name;
+	gulong type;
+
+	type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	g_assert (type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST);
 
 	status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
 	file_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
@@ -489,7 +489,8 @@ rb_podcast_manager_update_feeds (RBPodcastManager *pd)
 {
 	GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(pd->priv->db));	
 
-	rhythmdb_do_full_query (pd->priv->db, query_model,
+	rhythmdb_do_full_query (pd->priv->db, 
+				RHYTHMDB_QUERY_RESULTS (query_model),
                                 RHYTHMDB_QUERY_PROP_EQUALS,
                                 RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
                                 RHYTHMDB_QUERY_END);
@@ -506,7 +507,7 @@ rb_podcast_manager_head_query_cb (GtkTreeModel *query_model,
  	   			  GtkTreePath *path, GtkTreeIter *iter,
 				  RBPodcastManager *manager)
 {
-        const char* uri;
+        const char *uri;
         RhythmDBEntry* entry;
 	guint status;
 
@@ -582,12 +583,6 @@ rb_podcast_manager_copy_post (RBPodcastManager *pd)
 		goto next_step;
 	}
 		
-	if (gnome_vfs_uri_is_local (remote_uri)) {	
-		rb_debug ("Error downloading podcast: uri is local");
-		goto next_step;
-	}
-
-
 	conf_dir_name = rb_podcast_manager_get_podcast_dir (pd);
 	dir_name = g_build_filename (conf_dir_name,
 				     album_name,
@@ -595,13 +590,13 @@ rb_podcast_manager_copy_post (RBPodcastManager *pd)
 	g_free (conf_dir_name);
 
 	 if (!g_file_test (dir_name, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		if (rb_podcast_manager_mkdir_with_parents (dir_name, 0750) != 0) {
+		if (!g_mkdir_with_parents (dir_name, 0750)) {
 			rb_debug ("Error downloading podcast: could not create local dirs");
 			goto next_step;
 		}
 	 }
 		 		 
-	short_name = gnome_vfs_uri_extract_short_name (remote_uri);	
+	short_name = gnome_vfs_uri_extract_short_name (remote_uri);
 	local_file_name = g_build_filename (dir_name,
 	 				    short_name,
 					   NULL);
@@ -636,6 +631,8 @@ rb_podcast_manager_copy_post (RBPodcastManager *pd)
 		} else if (remote_size == info->size) {
 			GValue val = {0,};
 			char *uri = gnome_vfs_uri_to_string (local_uri, GNOME_VFS_URI_HIDE_NONE);
+			char *canon_uri = rb_canonicalise_uri (uri);
+			g_free (uri);
 		
 			rb_debug ("podcast %s already downloaded", location);
 
@@ -645,12 +642,13 @@ rb_podcast_manager_copy_post (RBPodcastManager *pd)
 			g_value_unset (&val);
 
 			g_value_init (&val, G_TYPE_STRING);
-			g_value_set_string (&val, uri);
+			g_value_set_string (&val, canon_uri);
 			rhythmdb_entry_set (pd->priv->db, data->entry, RHYTHMDB_PROP_MOUNTPOINT, &val);
 			g_value_unset (&val);
 
-			rb_podcast_manager_save_metadata (pd->priv->db, data->entry, uri);
+			rb_podcast_manager_save_metadata (pd->priv->db, data->entry, canon_uri);
 			rhythmdb_commit (pd->priv->db);
+			g_free (canon_uri);
 				
 			goto next_step;
 		} else if (remote_size > info->size) {
@@ -689,64 +687,8 @@ next_step:
 	gtk_idle_add ((GtkFunction) rb_podcast_manager_next_file , pd);	
 }
 
-
-static int
-rb_podcast_manager_mkdir_with_parents (const gchar *pathname,
-		   			int          mode)
-{
-  gchar *fn, *p;
-
-  if (pathname == NULL || *pathname == '\0')
-    {
-      return -1;
-    }
-
-  fn = g_strdup (pathname);
-
-  if (g_path_is_absolute (fn))
-    p = (gchar *) g_path_skip_root (fn);
-  else
-    p = fn;
-
-  do
-    {
-      while (*p && !G_IS_DIR_SEPARATOR (*p))
-	p++;
-      
-      if (!*p)
-	p = NULL;
-      else
-	*p = '\0';
-      
-      if (!g_file_test (fn, G_FILE_TEST_EXISTS))
-	{
-	  if (g_mkdir (fn, mode) == -1)
-	    {
-	      g_free (fn);
-	      return -1;
-	    }
-	}
-      else if (!g_file_test (fn, G_FILE_TEST_IS_DIR))
-	{
-	  g_free (fn);
-	  return -1;
-	}
-      if (p)
-	{
-	  *p++ = G_DIR_SEPARATOR;
-	  while (*p && G_IS_DIR_SEPARATOR (*p))
-	    p++;
-	}
-    }
-  while (p);
-
-  g_free (fn);
-
-  return 0;
-}
-
 gboolean
-rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char* url)
+rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char *url)
 {
 	RBPodcastThreadInfo *info;
 	gchar *valid_url = gnome_vfs_make_uri_from_input (url);
@@ -785,20 +727,20 @@ rb_podcast_manager_thread_parse_feed (RBPodcastThreadInfo *info)
 	RBPodcastManagerEvent *event = g_new0 (RBPodcastManagerEvent, 1);
 	RBPodcastChannel *feed = g_new0 (RBPodcastChannel, 1);
 
-	rb_podcast_parse_load_feed (feed, info->url);	
-	
-	event->channel = feed;
-	event->type = (feed->title == NULL) ? EVENT_ERROR_FEED : EVENT_INSERT_FEED;
+	if (rb_podcast_parse_load_feed (feed, info->url)) {
+		event->channel = feed;
+		event->type = (feed->title == NULL) ? EVENT_ERROR_FEED : EVENT_INSERT_FEED;
 
-	g_async_queue_push (info->pd->priv->event_queue, event);
-	g_idle_add ((GSourceFunc) rb_podcast_manager_event_loop, info->pd);
+		g_async_queue_push (info->pd->priv->event_queue, event);
+		g_idle_add ((GSourceFunc) rb_podcast_manager_event_loop, info->pd);
+	}
 	
 	g_free (info->url);
 	g_free (info);
 	return NULL;
 }
 
-gboolean
+RhythmDBEntry *
 rb_podcast_manager_add_post (RhythmDB *db,
 			      const char *name,
 			      const char *title,
@@ -806,102 +748,100 @@ rb_podcast_manager_add_post (RhythmDB *db,
 			      const char *generator,
 			      const char *uri,
 			      const char *description,
-			      gulong status,
 			      gulong date,
 			      gulong duration,
 			      guint64 filesize)
 {
+	RhythmDBEntry *entry;
+	GValue val = {0,};
+	GTimeVal time;
 
-	if (uri && name && title && date && g_utf8_validate(uri, -1, NULL)) {
-		RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (db, uri);
-		GValue val = {0,};
-		GTimeVal time;
-
-		if (entry)
-			return FALSE;
-
-		entry = rhythmdb_entry_new (db,
-					    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
-				    	    uri);
-		if (entry == NULL)
-			return FALSE;
-
-		g_value_init (&val, G_TYPE_STRING);
-		g_value_set_string (&val, name);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &val);
-		
-		g_value_reset (&val);
-		g_value_set_static_string (&val, _("Podcast"));
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &val);
-
-		g_value_reset (&val);
-		g_value_set_string (&val, title);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &val);
-
-		g_value_reset (&val);
-		if (subtitle)
-			g_value_set_string (&val, subtitle);
-		else
-			g_value_set_static_string (&val, "");
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &val);
-
-		g_value_reset (&val);
-		if (description)
-			g_value_set_string (&val, description);
-		else
-			g_value_set_static_string (&val, "");
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DESCRIPTION, &val);
-
-		g_value_reset (&val);
-		if (generator) 
-			g_value_set_string (&val, generator);
-		else
-			g_value_set_static_string (&val, "");
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &val);
-		g_value_unset (&val);
-
-		g_value_init (&val, G_TYPE_ULONG);
-		g_value_set_ulong (&val, status);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_STATUS, &val);
-
-		g_value_reset (&val);
-		g_value_set_ulong (&val, date);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_POST_TIME, &val);
-
-		g_value_reset (&val);
-		g_value_set_ulong (&val, duration);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DURATION, &val);
-	
-		g_value_reset (&val);
-		g_value_set_ulong (&val, 0);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &val);
-
-		/* first seen */
-		g_get_current_time (&time);
-		g_value_reset (&val);
-		g_value_set_ulong (&val, time.tv_sec);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FIRST_SEEN, &val);
-		g_value_unset (&val);
-		
-		/* initialize the rating */
-		g_value_init (&val, G_TYPE_DOUBLE);
-		g_value_set_double (&val, 2.5);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_RATING, &val);
-		g_value_unset (&val);
-
-		g_value_init (&val, G_TYPE_UINT64);
-		g_value_set_uint64 (&val, filesize);
-		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FILE_SIZE, &val);
-		g_value_unset (&val);
-
-		return TRUE;
-	} else {
-		return FALSE;
+	if (!uri || !name || !title || !date || !g_utf8_validate(uri, -1, NULL)) {
+		return NULL;
 	}
+	entry = rhythmdb_entry_lookup_by_location (db, uri);
+	if (entry)
+		return NULL;
+
+	entry = rhythmdb_entry_new (db,
+				    RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
+				    uri);
+	if (entry == NULL)
+		return NULL;
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, name);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ALBUM, &val);
+	
+	g_value_reset (&val);
+	g_value_set_static_string (&val, _("Podcast"));
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_GENRE, &val);
+
+	g_value_reset (&val);
+	g_value_set_string (&val, title);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &val);
+
+	g_value_reset (&val);
+	if (subtitle)
+		g_value_set_string (&val, subtitle);
+	else
+		g_value_set_static_string (&val, "");
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &val);
+
+	g_value_reset (&val);
+	if (description)
+		g_value_set_string (&val, description);
+	else
+		g_value_set_static_string (&val, "");
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DESCRIPTION, &val);
+
+	g_value_reset (&val);
+	if (generator) 
+		g_value_set_string (&val, generator);
+	else
+		g_value_set_static_string (&val, "");
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &val);
+	g_value_unset (&val);
+
+	g_value_init (&val, G_TYPE_ULONG);
+	g_value_set_ulong (&val, RHYTHMDB_PODCAST_STATUS_PAUSED);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_STATUS, &val);
+
+	g_value_reset (&val);
+	g_value_set_ulong (&val, date);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_POST_TIME, &val);
+
+	g_value_reset (&val);
+	g_value_set_ulong (&val, duration);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DURATION, &val);
+
+	g_value_reset (&val);
+	g_value_set_ulong (&val, 0);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &val);
+
+	/* first seen */
+	g_get_current_time (&time);
+	g_value_reset (&val);
+	g_value_set_ulong (&val, time.tv_sec);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FIRST_SEEN, &val);
+	g_value_unset (&val);
+	
+	/* initialize the rating */
+	g_value_init (&val, G_TYPE_DOUBLE);
+	g_value_set_double (&val, 2.5);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_RATING, &val);
+	g_value_unset (&val);
+
+	g_value_init (&val, G_TYPE_UINT64);
+	g_value_set_uint64 (&val, filesize);
+	rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_FILE_SIZE, &val);
+	g_value_unset (&val);
+
+	return entry;
 }
 
 static gboolean
-rb_podcast_manager_save_metadata (RhythmDB *db, RhythmDBEntry *entry, const char* uri)
+rb_podcast_manager_save_metadata (RhythmDB *db, RhythmDBEntry *entry, const char *uri)
 {
 	RBMetaData *md = rb_metadata_new();
 	GError *error = NULL;
@@ -926,6 +866,8 @@ rb_podcast_manager_save_metadata (RhythmDB *db, RhythmDBEntry *entry, const char
 		rhythmdb_commit (db);
 		
 		g_object_unref (md);
+		g_error_free (error);
+
 		return FALSE;
 	}
 
@@ -961,7 +903,8 @@ static void
 rb_podcast_manager_db_entry_added_cb (RBPodcastManager *pd, RhythmDBEntry *entry)
 {
 
-	if (entry->type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
+	gulong type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+	if (type != RHYTHMDB_ENTRY_TYPE_PODCAST_POST)
 		return;
 	
         rb_podcast_manager_download_entry (pd, entry);
@@ -1165,13 +1108,16 @@ download_progress_cb (GnomeVFSXferProgressInfo *info, gpointer cb_data)
 	if (rhythmdb_entry_get_string (data->entry, RHYTHMDB_PROP_MOUNTPOINT) == NULL) {
 		GValue val = {0,};
 		RhythmDB *db = data->pd->priv->db;
+		char *uri = gnome_vfs_uri_to_string (data->write_uri, GNOME_VFS_URI_HIDE_NONE);
+		char *canon_uri = rb_canonicalise_uri (uri);
+		g_free (uri);
 
 		g_value_init (&val, G_TYPE_STRING);
-		g_value_set_string (&val,  
-				    gnome_vfs_uri_to_string (data->write_uri, GNOME_VFS_URI_HIDE_NONE));
+		g_value_set_string (&val, canon_uri);
 		rhythmdb_entry_set (db, data->entry, RHYTHMDB_PROP_MOUNTPOINT, &val);
 		g_value_unset (&val);
 		rhythmdb_commit (db);
+		g_free (canon_uri);
 	}
 	
 	if (info->phase  == GNOME_VFS_XFER_PHASE_COMPLETED) {
@@ -1214,7 +1160,13 @@ download_progress_update_cb (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressIn
 			local_progress = (gint) 100 * info->total_bytes_copied / info->file_size;
 
 		if (local_progress != data->progress) {
-			data->entry->podcast->status = local_progress;
+			GValue val = {0,};
+
+			g_value_init (&val, G_TYPE_ULONG);
+			g_value_set_ulong (&val, local_progress);
+			rhythmdb_entry_set (data->pd->priv->db, data->entry, RHYTHMDB_PROP_STATUS, &val);
+			g_value_unset (&val);
+
 			GDK_THREADS_ENTER ();
 			
 			g_signal_emit (data->pd, rb_podcast_manager_signals[STATUS_CHANGED],
@@ -1223,14 +1175,13 @@ download_progress_update_cb (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressIn
 			GDK_THREADS_LEAVE ();
 			data->progress = local_progress;
 		}
-		
 	}
 	
 	return GNOME_VFS_XFER_ERROR_ACTION_SKIP;
 }
 
 void
-rb_podcast_manager_unsubscribe_feed (RhythmDB *db, const char* url)
+rb_podcast_manager_unsubscribe_feed (RhythmDB *db, const char *url)
 {
 	RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (db, url);
 	if (entry) {
@@ -1244,10 +1195,13 @@ rb_podcast_manager_unsubscribe_feed (RhythmDB *db, const char* url)
 }
 
 gboolean
-rb_podcast_manager_remove_feed (RBPodcastManager *pd, const char* url, gboolean remove_files)
+rb_podcast_manager_remove_feed (RBPodcastManager *pd, const char *url, gboolean remove_files)
 {
 	RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (pd->priv->db, url);
+
 	if (entry) {
+		rb_debug ("Removing podcast feed: %s remove_files: %d", url, remove_files);
+
 		rb_podcast_manager_set_remove_files (pd, remove_files);
 		rhythmdb_entry_delete (pd->priv->db, entry);
 		rhythmdb_commit (pd->priv->db);
@@ -1258,15 +1212,19 @@ rb_podcast_manager_remove_feed (RBPodcastManager *pd, const char* url, gboolean
 }
 
 static void 
-rb_podcast_manager_db_entry_deleted_cb (RBPodcastManager *pd, RhythmDBEntry *entry)
+rb_podcast_manager_db_entry_deleted_cb (RBPodcastManager *pd,
+					RhythmDBEntry *entry)
 {
 
-	if ( (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) && (pd->priv->remove_files == TRUE) )
-	{
-		const gchar *file_name;
-		const gchar *dir_name;
-		const gchar *conf_dir_name;
-		GnomeVFSURI *uri;
+	gulong type = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE);
+
+	if ((type == RHYTHMDB_ENTRY_TYPE_PODCAST_POST) && (pd->priv->remove_files == TRUE)) {
+		const char *file_name;
+		const char *dir_name;
+		const char *conf_dir_name;
+		GnomeVFSResult result;
+
+		rb_debug ("Handling entry deleted");
 
 		/* make sure we're not downloading it */
 		rb_podcast_manager_cancel_download (pd, entry);
@@ -1274,30 +1232,31 @@ rb_podcast_manager_db_entry_deleted_cb (RBPodcastManager *pd, RhythmDBEntry *ent
 		file_name = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT);
 		if (file_name == NULL) {
 			/* episode has not been downloaded */
+			rb_debug ("Episode not downloaded, skipping.");
 			return;
 		}
-		
-		uri = gnome_vfs_uri_new (file_name);
-		
-		if ((uri != NULL) && (gnome_vfs_uri_is_local (uri) == TRUE)) {
-			gnome_vfs_unlink (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_MOUNTPOINT));
-		
-			/* remove dir */
-			rb_debug ("removing dir");
-			conf_dir_name = eel_gconf_get_string (CONF_STATE_PODCAST_DOWNLOAD_DIR);
-			
-			dir_name = g_build_filename (conf_dir_name,
-						     rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM),
-						     NULL);
-			gnome_vfs_remove_directory (dir_name);
+
+		result = gnome_vfs_unlink (file_name);
+		if (result != GNOME_VFS_OK) {
+			rb_debug ("Removing episode failed: %s", gnome_vfs_result_to_string (result));
+			return;
 		}
-	}
-	else if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
-	{
+
+		/* remove dir */
+		rb_debug ("removing dir");
+		conf_dir_name = eel_gconf_get_string (CONF_STATE_PODCAST_DOWNLOAD_DIR);
+			
+		dir_name = g_build_filename (conf_dir_name,
+					     rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM),
+					     NULL);
+		gnome_vfs_remove_directory (dir_name);
+
+	} else if (type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
 		GtkTreeModel* query_model = GTK_TREE_MODEL (rhythmdb_query_model_new_empty(pd->priv->db));	
 		GtkTreeIter iter;
 
-		rhythmdb_do_full_query (pd->priv->db, query_model,
+		rhythmdb_do_full_query (pd->priv->db, 
+					RHYTHMDB_QUERY_RESULTS (query_model),
                 	                RHYTHMDB_QUERY_PROP_EQUALS,
                         	        RHYTHMDB_PROP_TYPE, RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
                 	                RHYTHMDB_QUERY_PROP_LIKE,
@@ -1371,10 +1330,11 @@ rb_podcast_manager_update_synctime (RBPodcastManager *pd)
 	rb_podcast_manager_start_sync (pd);
 }
 
-static void  rb_podcast_manager_config_changed (GConfClient* client,
-                                        	guint cnxn_id,
-				                GConfEntry *entry,
-						gpointer user_data)
+static void
+rb_podcast_manager_config_changed (GConfClient* client,
+				   guint cnxn_id,
+				   GConfEntry *entry,
+				   gpointer user_data)
 {
 	rb_podcast_manager_update_synctime (RB_PODCAST_MANAGER (user_data));
 }
@@ -1409,7 +1369,8 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	GValue last_update_val = { 0, };
 	gulong last_post = 0;
 	gulong new_last_post;
-	gboolean new_feed, updated;
+	GList *download_entries = NULL;
+	gboolean new_feed, updated, download_last;
 	RhythmDB *db = pd->priv->db;
 
 	RhythmDBEntry *entry;
@@ -1425,7 +1386,7 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	new_feed = TRUE;
 
 	/* processing podcast head */
-	entry = rhythmdb_entry_lookup_by_location (db, (gchar* )data->url);
+	entry = rhythmdb_entry_lookup_by_location (db, (gchar *)data->url);
 	if (entry) {
 		if (rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TYPE) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED)
 			return;
@@ -1441,19 +1402,19 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 		rb_debug ("Insert new entry");
 		entry = rhythmdb_entry_new (db,
 					    RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
-				    	    (gchar*) data->url);
+				    	    (gchar *) data->url);
 		if (entry == NULL)
 			return;
 		rb_debug("New entry create\n");
 	
 		g_value_init (&title_val, G_TYPE_STRING);
-		g_value_set_string (&title_val, (gchar* ) data->title);
+		g_value_set_string (&title_val, (gchar * ) data->title);
 		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_TITLE, &title_val);
 		g_value_unset (&title_val);
 	
 		g_value_init (&author_val, G_TYPE_STRING);
 		if (data->author)
-			g_value_set_string (&author_val, (gchar* ) data->author);
+			g_value_set_string (&author_val, (gchar *) data->author);
 		else
 			g_value_set_static_string (&author_val, _("Unknown"));
 		rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_ARTIST, &author_val);
@@ -1462,42 +1423,42 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	
 		if (data->subtitle) {
 			g_value_init (&subtitle_val, G_TYPE_STRING);
-			g_value_set_string (&subtitle_val, (gchar* ) data->subtitle);
+			g_value_set_string (&subtitle_val, (gchar *) data->subtitle);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUBTITLE, &subtitle_val);
 			g_value_unset (&subtitle_val);
 		}
 	
 		if (data->description) {
 			g_value_init (&description_val, G_TYPE_STRING);
-			g_value_set_string (&description_val, (gchar* ) data->description);
+			g_value_set_string (&description_val, (gchar *) data->description);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_DESCRIPTION, &description_val);
 			g_value_unset (&description_val);
 		}
 	
 		if (data->summary) {
 			g_value_init (&summary_val, G_TYPE_STRING);
-			g_value_set_string (&summary_val, (gchar* ) data->summary);
+			g_value_set_string (&summary_val, (gchar *) data->summary);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_SUMMARY, &summary_val);
 			g_value_unset (&summary_val);
 		}
 
 		if (data->lang) {
 			g_value_init (&lang_val, G_TYPE_STRING);
-			g_value_set_string (&lang_val, (gchar* ) data->lang);
+			g_value_set_string (&lang_val, (gchar *) data->lang);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_LANG, &lang_val);
 			g_value_unset (&lang_val);
 		}
 
 		if (data->copyright) { 
 			g_value_init (&copyright_val, G_TYPE_STRING);
-			g_value_set_string (&copyright_val, (gchar* ) data->copyright);
+			g_value_set_string (&copyright_val, (gchar *) data->copyright);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_COPYRIGHT, &copyright_val);
 			g_value_unset (&copyright_val);
 		}
 
 		if (data->img) {
 			g_value_init (&image_val, G_TYPE_STRING);
-			g_value_set_string (&image_val, (gchar* ) data->img);
+			g_value_set_string (&image_val, (gchar *) data->img);
 			rhythmdb_entry_set_uninserted (db, entry, RHYTHMDB_PROP_IMAGE, &image_val);
 			g_value_unset (&image_val);
 		}
@@ -1514,35 +1475,54 @@ rb_podcast_manager_insert_feed (RBPodcastManager *pd, RBPodcastChannel *data)
 	new_last_post = last_post;
 	
 	updated = FALSE;
+	download_last = (eel_gconf_get_integer (CONF_STATE_PODCAST_DOWNLOAD_INTERVAL) != UPDATE_MANUALLY);
 	for (lst_songs = data->posts; lst_songs != NULL; lst_songs = g_list_next (lst_songs)) {
 		RBPodcastItem *item = (RBPodcastItem *) lst_songs->data;
+		RhythmDBEntry *post_entry;
 
 		if (item->pub_date > last_post || item->pub_date == 0) {
-			gulong status;
 			updated = TRUE;
 
-			/* last episode gets status RHYTHMDB_PODCAST_STATUS_WAITING, so that it begins downloading */
-			if (lst_songs == (g_list_last (data->posts)))
-				status = RHYTHMDB_PODCAST_STATUS_WAITING;
-			else
-				status = RHYTHMDB_PODCAST_STATUS_PAUSED;
-			
-			rb_podcast_manager_add_post (db, 
-						     (gchar*) data->title,
-						     (gchar*) item->title, 
-					 	     (gchar*) data->url,
-						     (gchar*) (item->author ? item->author : data->author),
-						     (gchar*) item->url,
-						     (gchar*) item->description,
-						     status,
-						     (gulong) (item->pub_date > 0 ? item->pub_date : data->pub_date),
-						     (gulong) item->duration,
-						     item->filesize);
-			if (item->pub_date > new_last_post)
+			post_entry = 
+				rb_podcast_manager_add_post (db, 
+							     (gchar *) data->title,
+							     (gchar *) item->title, 
+							     (gchar *) data->url,
+							     (gchar *) (item->author ? item->author : data->author),
+							     (gchar *) item->url,
+							     (gchar *) item->description,
+							     (gulong) (item->pub_date > 0 ? item->pub_date : data->pub_date),
+							     (gulong) item->duration,
+							     item->filesize);
+			if (item->pub_date >= new_last_post) {
+				if (item->pub_date > new_last_post) {
+					g_list_free (download_entries);
+					download_entries = NULL;
+				}
+				download_entries = g_list_prepend (download_entries, post_entry);
 				new_last_post = item->pub_date;
+			}
 		}
 	}
 
+	if (download_last) {
+		GValue status = {0,};
+		GList *t;
+
+		g_value_init (&status, G_TYPE_ULONG);
+		g_value_set_ulong (&status, RHYTHMDB_PODCAST_STATUS_WAITING);
+		t = download_entries;
+		while (t) {
+			rhythmdb_entry_set_uninserted (db, 
+						       (RhythmDBEntry*) t->data,
+						       RHYTHMDB_PROP_STATUS, 
+						       &status);
+			t = t->next;
+		}
+		g_value_unset (&status);
+	}
+	g_list_free (download_entries);
+
 	if (updated)
 		g_signal_emit (pd, rb_podcast_manager_signals[FEED_UPDATES_AVALIABLE],
 			       0, entry);
@@ -1587,7 +1567,7 @@ rb_podcast_manager_event_loop (RBPodcastManager *pd)
 			{
 				gchar *error_msg;
 				error_msg = g_strdup_printf (_("There was a problem adding this podcast. Please verify the URL: %s"),
-							     (gchar*) event->channel->url);
+							     (gchar *) event->channel->url);
 				g_signal_emit (G_OBJECT (pd), 
 					       rb_podcast_manager_signals[PROCESS_ERROR], 
 					       0, error_msg);
