@@ -33,14 +33,17 @@
 #include "rb-debug.h"
 #include "gsequence.h"
 #include "rb-tree-dnd.h"
-#include "rhythmdb-marshal.h"
+#include "rb-marshal.h"
 #include "rb-util.h"
 
+static void rhythmdb_query_model_query_results_init (RhythmDBQueryResultsIface *iface);
 static void rhythmdb_query_model_tree_model_init (GtkTreeModelIface *iface);
 static void rhythmdb_query_model_drag_source_init (RbTreeDragSourceIface *iface);
 static void rhythmdb_query_model_drag_dest_init (RbTreeDragDestIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE(RhythmDBQueryModel, rhythmdb_query_model, G_TYPE_OBJECT,
+			G_IMPLEMENT_INTERFACE(RHYTHMDB_TYPE_QUERY_RESULTS,
+					      rhythmdb_query_model_query_results_init)
 			G_IMPLEMENT_INTERFACE(GTK_TYPE_TREE_MODEL,
 					      rhythmdb_query_model_tree_model_init)
 			G_IMPLEMENT_INTERFACE(RB_TYPE_TREE_DRAG_SOURCE,
@@ -51,6 +54,7 @@ G_DEFINE_TYPE_WITH_CODE(RhythmDBQueryModel, rhythmdb_query_model, G_TYPE_OBJECT,
 static void rhythmdb_query_model_init (RhythmDBQueryModel *shell_player);
 static GObject *rhythmdb_query_model_constructor (GType type, guint n_construct_properties,
 						  GObjectConstructParam *construct_properties);
+static void rhythmdb_query_model_dispose (GObject *object);
 static void rhythmdb_query_model_finalize (GObject *object);
 static void rhythmdb_query_model_set_property (GObject *object,
 					       guint prop_id,
@@ -72,8 +76,8 @@ static void rhythmdb_query_model_entry_deleted_cb (RhythmDB *db, RhythmDBEntry *
 
 static void rhythmdb_query_model_filter_out_entry (RhythmDBQueryModel *model,
 						   RhythmDBEntry *entry);
-static void rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry);
-static void rhythmdb_query_model_emit_reorder (RhythmDBQueryModel *model, gint old_pos, gint new_pos);
+static gboolean rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry);
+static gboolean rhythmdb_query_model_emit_reorder (RhythmDBQueryModel *model, gint old_pos, gint new_pos);
 static gboolean rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 							  GList *paths,
 							  GtkSelectionData *selection_data);
@@ -94,6 +98,10 @@ static gboolean rhythmdb_query_model_row_drop_position (RbTreeDragDest   *drag_d
 							GList *targets,
 							GtkTreeViewDropPosition *pos);
 
+static void rhythmdb_query_model_set_query (RhythmDBQueryResults *results, GPtrArray *query);
+static void rhythmdb_query_model_add_results (RhythmDBQueryResults *results, GPtrArray *entries);
+static void rhythmdb_query_model_query_complete (RhythmDBQueryResults *results);
+
 static GtkTreeModelFlags rhythmdb_query_model_get_flags (GtkTreeModel *model);
 static gint rhythmdb_query_model_get_n_columns (GtkTreeModel *tree_model);
 static GType rhythmdb_query_model_get_column_type (GtkTreeModel *tree_model, int index);
@@ -133,17 +141,14 @@ static void rhythmdb_query_model_base_non_entry_dropped (GtkTreeModel *base_mode
 							 RhythmDBQueryModel *model);
 static void rhythmdb_query_model_base_complete (GtkTreeModel *base_model,
 						RhythmDBQueryModel *model);
-static void rhythmdb_query_model_base_entry_prop_changed (GtkTreeModel *base_model,
-							  RhythmDBEntry *entry,
-							  RhythmDBPropType prop,
-							  const GValue *old,
-							  const GValue *new,
-							  RhythmDBQueryModel *model);
 static void rhythmdb_query_model_base_rows_reordered (GtkTreeModel *base_model,
 						      GtkTreePath *arg1,
 						      GtkTreeIter *arg2,
 						      gint *order_map,
 						      RhythmDBQueryModel *model);
+static void rhythmdb_query_model_base_entry_removed (RhythmDBQueryModel *base_model,
+						     RhythmDBEntry *entry,
+						     RhythmDBQueryModel *model);
 static int rhythmdb_query_model_child_index_to_base_index (RhythmDBQueryModel *model, int index);
 
 static gint _reverse_sorting_func (gpointer a, gpointer b, RhythmDBQueryModel *model);
@@ -161,9 +166,9 @@ struct RhythmDBQueryModelUpdate
 		struct {
 			RhythmDBEntry *entry;
 			gint index;
-		};
+		} data;
 		GPtrArray *entries;
-	};
+	} entrydata;
 };
 
 static void rhythmdb_query_model_process_update (struct RhythmDBQueryModelUpdate *update);
@@ -189,12 +194,12 @@ struct RhythmDBQueryModelPrivate
 
 	guint stamp;
 
-	GnomeVFSFileSize max_size;
+	guint64 max_size;
 	guint max_count;
 	guint max_time;
 
 	glong total_duration;
-	GnomeVFSFileSize total_size;
+	guint64 total_size;
 
 	GSequence *entries;
 	GHashTable *reverse_map;
@@ -248,6 +253,7 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 	object_class->set_property = rhythmdb_query_model_set_property;
 	object_class->get_property = rhythmdb_query_model_get_property;
 
+	object_class->dispose = rhythmdb_query_model_dispose;
 	object_class->finalize = rhythmdb_query_model_finalize;
 	object_class->constructor = rhythmdb_query_model_constructor;
 
@@ -327,24 +333,25 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, entry_prop_changed),
 			      NULL, NULL,
-			      rhythmdb_marshal_VOID__POINTER_INT_POINTER_POINTER,
-			      G_TYPE_NONE, 4, G_TYPE_POINTER,
-			      G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
+			      rb_marshal_VOID__POINTER_INT_POINTER_POINTER,
+			      G_TYPE_NONE,
+			      4, RHYTHMDB_TYPE_ENTRY, G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
 	rhythmdb_query_model_signals[ENTRY_REMOVED] =
 		g_signal_new ("entry-removed",
 			      RHYTHMDB_TYPE_QUERY_MODEL,
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, entry_removed),
 			      NULL, NULL,
-			      rhythmdb_marshal_VOID__POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_POINTER);
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1, RHYTHMDB_TYPE_ENTRY);
 	rhythmdb_query_model_signals[NON_ENTRY_DROPPED] =
 		g_signal_new ("non-entry-dropped",
 			      RHYTHMDB_TYPE_QUERY_MODEL,
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RhythmDBQueryModelClass, non_entry_dropped),
 			      NULL, NULL,
-			      rhythmdb_marshal_VOID__POINTER_INT,
+			      rb_marshal_VOID__POINTER_INT,
 			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_INT);
 	rhythmdb_query_model_signals[COMPLETE] =
 		g_signal_new ("complete",
@@ -358,6 +365,14 @@ rhythmdb_query_model_class_init (RhythmDBQueryModelClass *klass)
 	g_type_class_add_private (klass, sizeof (RhythmDBQueryModelPrivate));
 }
 
+static void
+rhythmdb_query_model_query_results_init (RhythmDBQueryResultsIface *iface)
+{
+	iface->set_query = rhythmdb_query_model_set_query;
+	iface->add_results = rhythmdb_query_model_add_results;
+	iface->query_complete = rhythmdb_query_model_query_complete;
+}
+
 static void
 rhythmdb_query_model_tree_model_init (GtkTreeModelIface *iface)
 {
@@ -445,17 +460,19 @@ rhythmdb_query_model_set_property (GObject *object,
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 							      G_CALLBACK (rhythmdb_query_model_base_complete),
 							      model);
-			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
-							      G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
-							      model);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 							      G_CALLBACK (rhythmdb_query_model_base_rows_reordered),
 							      model);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
+							      G_CALLBACK (rhythmdb_query_model_base_entry_removed),
+							      model);
+			g_object_unref (model->priv->base_model);
 		}
 
 		model->priv->base_model = g_value_get_object (value);
 
 		if (model->priv->base_model) {
+			g_object_ref (model->priv->base_model);
 			g_assert (model->priv->base_model->priv->db == model->priv->db);
 
 			g_signal_connect_object (G_OBJECT (model->priv->base_model),
@@ -474,14 +491,14 @@ rhythmdb_query_model_set_property (GObject *object,
 						 "complete",
 						 G_CALLBACK (rhythmdb_query_model_base_complete),
 						 model, 0);
-			g_signal_connect_object (G_OBJECT (model->priv->base_model),
-						 "entry-prop-changed",
-						 G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
-						 model, 0);
 			g_signal_connect_object (G_OBJECT (model->priv->base_model),
 						 "rows-reordered",
 						 G_CALLBACK (rhythmdb_query_model_base_rows_reordered),
 						 model, 0);
+			g_signal_connect_object (G_OBJECT (model->priv->base_model),
+						 "entry-removed",
+						 G_CALLBACK (rhythmdb_query_model_base_entry_removed),
+						 model, 0);
 
 			if (model->priv->base_model->priv->entries) {
 				GSequencePtr ptr, next;
@@ -493,7 +510,8 @@ rhythmdb_query_model_set_property (GObject *object,
 					entry = (RhythmDBEntry *)g_sequence_ptr_get_data (ptr);
 					if (model->priv->query == NULL ||
 					    rhythmdb_evaluate_query (model->priv->db, model->priv->query, entry)) {
-						rhythmdb_query_model_do_insert (model, entry, -1);
+						if (model->priv->show_hidden || (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN) == FALSE))
+							rhythmdb_query_model_do_insert (model, entry, -1);
 					}
 
 					ptr = next;
@@ -574,22 +592,14 @@ rhythmdb_query_model_constructor (GType type, guint n_construct_properties,
 				  GObjectConstructParam *construct_properties)
 {
 	RhythmDBQueryModel *model;
-	RhythmDBQueryModelClass *klass;
-	GObjectClass *parent_class;  
 
-	klass = RHYTHMDB_QUERY_MODEL_CLASS (g_type_class_peek (RHYTHMDB_TYPE_QUERY_MODEL));
+	model = RHYTHMDB_QUERY_MODEL (G_OBJECT_CLASS (rhythmdb_query_model_parent_class)->
+			constructor (type, n_construct_properties, construct_properties));
 
-	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-	model = RHYTHMDB_QUERY_MODEL (parent_class->constructor (type, n_construct_properties,
-								 construct_properties));
 	g_signal_connect_object (G_OBJECT (model->priv->db),
 				 "entry_added",
 				 G_CALLBACK (rhythmdb_query_model_entry_added_cb),
 				 model, 0);
-	g_signal_connect_object (G_OBJECT (model->priv->db),
-				 "entry_restored",
-				 G_CALLBACK (rhythmdb_query_model_entry_added_cb),
-				 model, 0);
 	g_signal_connect_object (G_OBJECT (model->priv->db),
 				 "entry_changed",
 				 G_CALLBACK (rhythmdb_query_model_entry_changed_cb),
@@ -609,7 +619,7 @@ _unref_entry (RhythmDBEntry *entry, gpointer stuff, RhythmDB *db)
 }
 
 static void
-rhythmdb_query_model_finalize (GObject *object)
+rhythmdb_query_model_dispose (GObject *object)
 {
 	RhythmDBQueryModel *model;
 
@@ -620,23 +630,9 @@ rhythmdb_query_model_finalize (GObject *object)
 
 	g_return_if_fail (model->priv != NULL);
 
-	rb_debug ("finalizing query model");
-
-	g_hash_table_foreach (model->priv->reverse_map, (GHFunc) _unref_entry, model->priv->db);
-	g_hash_table_foreach (model->priv->limited_reverse_map, (GHFunc) _unref_entry, model->priv->db);
+	rb_debug ("disposing query model %p", object);
 		
-	g_hash_table_destroy (model->priv->reverse_map);
-	g_sequence_free (model->priv->entries);
-	g_hash_table_destroy (model->priv->limited_reverse_map);
-	g_sequence_free (model->priv->limited_entries);
-
-	if (model->priv->query)
-		rhythmdb_query_free (model->priv->query);
-	if (model->priv->original_query)
-		rhythmdb_query_free (model->priv->original_query);
-
 	if (model->priv->base_model) {
-		g_object_unref (G_OBJECT (model->priv->base_model));
 		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 						      G_CALLBACK (rhythmdb_query_model_base_row_inserted),
 						      model);
@@ -649,14 +645,44 @@ rhythmdb_query_model_finalize (GObject *object)
 		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 						      G_CALLBACK (rhythmdb_query_model_base_complete),
 						      model);
-		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
-						      G_CALLBACK (rhythmdb_query_model_base_entry_prop_changed),
-						      model);
 		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
 						      G_CALLBACK (rhythmdb_query_model_base_rows_reordered),
 						      model);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->base_model),
+						      G_CALLBACK (rhythmdb_query_model_base_entry_removed),
+						      model);
+		g_object_unref (G_OBJECT (model->priv->base_model));
+		model->priv->base_model = NULL;
 	}
+	
+	G_OBJECT_CLASS (rhythmdb_query_model_parent_class)->dispose (object);
+}
 
+static void
+rhythmdb_query_model_finalize (GObject *object)
+{
+	RhythmDBQueryModel *model;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (RHYTHMDB_IS_QUERY_MODEL (object));
+
+	model = RHYTHMDB_QUERY_MODEL (object);
+
+	g_return_if_fail (model->priv != NULL);
+
+	rb_debug ("finalizing query model %p", object);
+
+	g_hash_table_foreach (model->priv->reverse_map, (GHFunc) _unref_entry, model->priv->db);
+	g_hash_table_destroy (model->priv->reverse_map);
+	g_sequence_free (model->priv->entries);
+	g_hash_table_foreach (model->priv->limited_reverse_map, (GHFunc) _unref_entry, model->priv->db);
+	g_hash_table_destroy (model->priv->limited_reverse_map);
+	g_sequence_free (model->priv->limited_entries);
+
+	if (model->priv->query)
+		rhythmdb_query_free (model->priv->query);
+	if (model->priv->original_query)
+		rhythmdb_query_free (model->priv->original_query);
 
 	G_OBJECT_CLASS (rhythmdb_query_model_parent_class)->finalize (object);
 }
@@ -686,19 +712,6 @@ rhythmdb_query_model_new_empty (RhythmDB *db)
 			     "db", db, NULL);
 }
 
-void
-rhythmdb_query_model_signal_complete (RhythmDBQueryModel *model)
-{
-	struct RhythmDBQueryModelUpdate *update;
-
-	update = g_new0 (struct RhythmDBQueryModelUpdate, 1);
-	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE;
-	update->model = model;
-	g_object_ref (G_OBJECT (model));
-
-	rhythmdb_query_model_process_update (update);
-}
-
 gboolean
 rhythmdb_query_model_has_pending_changes (RhythmDBQueryModel *model)
 {
@@ -759,7 +772,8 @@ rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 		g_value_init (&false_val, G_TYPE_BOOLEAN);
 		g_value_set_boolean (&false_val, FALSE);
 		
-		rb_debug ("emitting hidden-removal notification for %s", entry->location);
+		rb_debug ("emitting hidden-removal notification for %s",
+			  rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 		g_signal_emit (G_OBJECT (model),
 			       rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
 			       entry, RHYTHMDB_PROP_HIDDEN, &false_val, &true_val);
@@ -793,7 +807,19 @@ rhythmdb_query_model_entry_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 	}
 
 	/* it may have moved, so we can't just emit a changed entry */
-	rhythmdb_query_model_do_reorder (model, entry);
+	if (!rhythmdb_query_model_do_reorder (model, entry)) {
+		/* but if it didn't, we can */
+		GtkTreeIter iter;
+		GtkTreePath *path;
+
+		if (rhythmdb_query_model_entry_to_iter (model, entry, &iter)) {
+			path = rhythmdb_query_model_get_path (GTK_TREE_MODEL (model),
+							      &iter);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (model),
+						     path, &iter);
+			gtk_tree_path_free (path);
+		}
+	}
 }
 
 static void
@@ -823,9 +849,9 @@ idle_process_update (struct RhythmDBQueryModelUpdate *update)
 	case RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED:
 	{
 		guint i;
-		rb_debug ("inserting %d rows", update->entries->len);
-		for (i = 0; i < update->entries->len; i++ ) {
-			RhythmDBEntry *entry = g_ptr_array_index (update->entries, i);
+		rb_debug ("inserting %d rows", update->entrydata.entries->len);
+		for (i = 0; i < update->entrydata.entries->len; i++ ) {
+			RhythmDBEntry *entry = g_ptr_array_index (update->entrydata.entries, i);
 
 			if (update->model->priv->show_hidden || !rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN)) {
 				RhythmDBQueryModel *base_model = update->model->priv->base_model;
@@ -838,14 +864,14 @@ idle_process_update (struct RhythmDBQueryModelUpdate *update)
 			
 			rhythmdb_entry_unref (update->model->priv->db, entry);
 		}
-		g_ptr_array_free (update->entries, TRUE);
+		g_ptr_array_free (update->entrydata.entries, TRUE);
 		break;
 	}
 	case RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED_INDEX:
 	{
-		rb_debug ("inserting row at index %d", update->index);
-		rhythmdb_query_model_do_insert (update->model, update->entry, update->index);
-		rhythmdb_entry_unref (update->model->priv->db, update->entry);
+		rb_debug ("inserting row at index %d", update->entrydata.data.index);
+		rhythmdb_query_model_do_insert (update->model, update->entrydata.data.entry, update->entrydata.data.index);
+		rhythmdb_entry_unref (update->model->priv->db, update->entrydata.data.entry);
 		break;
 	}
 	case RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE:
@@ -859,29 +885,6 @@ idle_process_update (struct RhythmDBQueryModelUpdate *update)
 	return FALSE;
 }
 
-/* Threading: Called from the database query thread for async queries,
- *  from the main thread for synchronous queries.
- */
-void
-rhythmdb_query_model_add_entries (RhythmDBQueryModel *model, GPtrArray *entries)
-{
-	struct RhythmDBQueryModelUpdate *update;
-	guint i;
-
-	rb_debug ("adding %d entries", entries->len);
-
-	update = g_new (struct RhythmDBQueryModelUpdate, 1);
-	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED;
-	update->entries = entries;
-	update->model = model;
-	g_object_ref (G_OBJECT (model));
-
-	for (i = 0; i < update->entries->len; i++)
-		rhythmdb_entry_ref (model->priv->db, g_ptr_array_index (update->entries, i));
-
-	rhythmdb_query_model_process_update (update);
-}
-
 void
 rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry, gint index)
 {
@@ -903,8 +906,8 @@ rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry,
 	
 	update = g_new (struct RhythmDBQueryModelUpdate, 1);
 	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROW_INSERTED_INDEX;
-	update->entry = entry;
-	update->index = index;
+	update->entrydata.data.entry = entry;
+	update->entrydata.data.index = index;
 	update->model = model;
 
 	g_object_ref (G_OBJECT (model));
@@ -912,7 +915,7 @@ rhythmdb_query_model_add_entry (RhythmDBQueryModel *model, RhythmDBEntry *entry,
 	rhythmdb_query_model_process_update (update);
 }
 
-GnomeVFSFileSize
+guint64
 rhythmdb_query_model_get_size (RhythmDBQueryModel *model)
 {
 	return model->priv->total_size;
@@ -995,8 +998,8 @@ rhythmdb_query_model_remove_from_main_list (RhythmDBQueryModel *model, RhythmDBE
 	gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
 	gtk_tree_path_free (path);
 	
-	model->priv->total_duration -= entry->duration;
-	model->priv->total_size -= entry->file_size;
+	model->priv->total_duration -= rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
+	model->priv->total_size -= rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
 
 	g_sequence_remove (ptr);
 	g_assert (g_hash_table_remove (model->priv->reverse_map, entry));
@@ -1046,8 +1049,8 @@ rhythmdb_query_model_update_limited_entries (RhythmDBQueryModel *model)
 		if (!entry)
 			break;
 
-		size = entry->file_size;
-		duration = entry->duration;
+		size = rhythmdb_entry_get_uint64 (entry, RHYTHMDB_PROP_FILE_SIZE);
+		duration = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DURATION);
 
 		if ((model->priv->max_count > 0 && (g_hash_table_size (model->priv->reverse_map) + 1) > model->priv->max_count) ||
 		    (model->priv->max_size > 0 && model->priv->total_size + size > model->priv->max_size) ||
@@ -1068,7 +1071,7 @@ rhythmdb_query_model_update_limited_entries (RhythmDBQueryModel *model)
 	}
 }
 
-static void
+static gboolean
 rhythmdb_query_model_emit_reorder (RhythmDBQueryModel *model, gint old_pos, gint new_pos)
 {
 	int length, i;
@@ -1078,7 +1081,7 @@ rhythmdb_query_model_emit_reorder (RhythmDBQueryModel *model, gint old_pos, gint
 
 	if (new_pos == old_pos) {
 		/* it hasn't moved, don't emit a re-order */
-		return;
+		return FALSE;
 	}
 
 	length = g_sequence_get_length (model->priv->entries);
@@ -1112,9 +1115,10 @@ rhythmdb_query_model_emit_reorder (RhythmDBQueryModel *model, gint old_pos, gint
 				       reorder_map);
 	gtk_tree_path_free (path);
 	free (reorder_map);
+	return TRUE;
 }
 
-static void
+static gboolean
 rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry)
 {
 	GSequencePtr ptr;
@@ -1124,7 +1128,7 @@ rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry
 	GCompareDataFunc sort_func = model->priv->sort_func;
 
 	if (sort_func == NULL)
-		return;
+		return FALSE;
 	if (model->priv->sort_reverse)
 		sort_func = (GCompareDataFunc) _reverse_sorting_func;
 
@@ -1138,7 +1142,7 @@ rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry
 			/* the entry belongs in the limited list, so we don't need a re-order */
 			rhythmdb_query_model_remove_entry (model, entry);
 			rhythmdb_query_model_do_insert (model, entry, -1);
-			return;
+			return TRUE;
 		}
 	}
 
@@ -1162,7 +1166,7 @@ rhythmdb_query_model_do_reorder (RhythmDBQueryModel *model, RhythmDBEntry *entry
 	new_pos = g_sequence_ptr_get_position (ptr);
 	g_hash_table_insert (model->priv->reverse_map, entry, ptr);
 
-	rhythmdb_query_model_emit_reorder (model, old_pos, new_pos);
+	return rhythmdb_query_model_emit_reorder (model, old_pos, new_pos);
 }
 
 static void
@@ -1421,7 +1425,7 @@ rhythmdb_query_model_drag_data_get (RbTreeDragSource *dragsource,
 
 			entry = g_sequence_ptr_get_data (iter.user_data);
 
-			location = entry->location;
+			location = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
 			g_string_append (data, location);
 
 			if (tem->next)
@@ -1520,6 +1524,8 @@ rhythmdb_query_model_drag_data_received (RbTreeDragDest *drag_dest,
 				GSequencePtr old_ptr = g_hash_table_lookup (model->priv->reverse_map,
 									    entry);
 
+				rhythmdb_entry_ref (model->priv->db, entry);
+
 				/* the entry already exists it is either a reorder drag and drop
 				   (or a drag and drop from another application), so we delete
 				   the existing one before adding it again. */
@@ -1587,6 +1593,51 @@ rhythmdb_query_model_row_drop_position (RbTreeDragDest   *drag_dest,
 	return TRUE;
 }
 
+static void
+rhythmdb_query_model_set_query (RhythmDBQueryResults *results, GPtrArray *query)
+{
+	g_object_set (G_OBJECT (results), "query", query, NULL);
+}
+
+/* Threading: Called from the database query thread for async queries,
+ *  from the main thread for synchronous queries.
+ */
+static void
+rhythmdb_query_model_add_results (RhythmDBQueryResults *results, GPtrArray *entries)
+{
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (results);
+	struct RhythmDBQueryModelUpdate *update;
+	guint i;
+
+	rb_debug ("adding %d entries", entries->len);
+
+	update = g_new (struct RhythmDBQueryModelUpdate, 1);
+	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_ROWS_INSERTED;
+	update->entrydata.entries = entries;
+	update->model = model;
+	g_object_ref (G_OBJECT (model));
+
+	for (i = 0; i < update->entrydata.entries->len; i++)
+		rhythmdb_entry_ref (model->priv->db, g_ptr_array_index (update->entrydata.entries, i));
+
+	rhythmdb_query_model_process_update (update);
+}
+
+static void
+rhythmdb_query_model_query_complete (RhythmDBQueryResults *results)
+{
+	RhythmDBQueryModel *model = RHYTHMDB_QUERY_MODEL (results);
+	struct RhythmDBQueryModelUpdate *update;
+
+	update = g_new0 (struct RhythmDBQueryModelUpdate, 1);
+	update->type = RHYTHMDB_QUERY_MODEL_UPDATE_QUERY_COMPLETE;
+	update->model = model;
+	g_object_ref (G_OBJECT (model));
+
+	rhythmdb_query_model_process_update (update);
+}
+
+
 static GtkTreeModelFlags
 rhythmdb_query_model_get_flags (GtkTreeModel *model)
 {
@@ -1771,6 +1822,43 @@ rhythmdb_query_model_compute_status_normal (RhythmDBQueryModel *model)
 					       rhythmdb_query_model_get_size (model));
 }
 
+static void
+apply_updated_entry_sequence (RhythmDBQueryModel *model, GSequence *new_entries)
+{
+	int *reorder_map;
+	int length, i;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GSequencePtr ptr;
+
+	length = g_sequence_get_length (new_entries);
+	/* generate resort map and rebuild reverse map */
+	reorder_map = malloc (length * sizeof(gint));
+
+	ptr = g_sequence_get_begin_ptr (new_entries);
+	for (i = 0; i < length; i++) {
+		gpointer entry = g_sequence_ptr_get_data (ptr);
+		GSequencePtr old_ptr;
+
+		old_ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
+		reorder_map[i] = g_sequence_ptr_get_position (ptr);
+		g_hash_table_replace (model->priv->reverse_map, entry, ptr);
+
+		ptr = g_sequence_ptr_next (ptr);
+	}
+
+	/* emit the re-order and clean up */
+	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter);
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
+				       path, &iter,
+				       reorder_map);
+
+	gtk_tree_path_free (path);
+	free (reorder_map);
+	g_sequence_free (model->priv->entries);
+	model->priv->entries = new_entries;
+}
 
 void
 rhythmdb_query_model_set_sort_order (RhythmDBQueryModel *model,
@@ -1780,11 +1868,13 @@ rhythmdb_query_model_set_sort_order (RhythmDBQueryModel *model,
 {
 	GSequence *new_entries;
 	GSequencePtr ptr;
-	GtkTreePath *path;
-	GtkTreeIter iter;
 	int length, i;
-	int *reorder_map;
 
+	if ((model->priv->sort_func == sort_func) &&
+	    (model->priv->sort_prop_id == sort_prop_id) &&
+	    (model->priv->sort_reverse == sort_reverse))
+		return;
+	
 	g_return_if_fail (((model->priv->max_count == 0) &&
 			  (model->priv->max_time == 0) &&
 			  (model->priv->max_size == 0)) ||
@@ -1813,32 +1903,7 @@ rhythmdb_query_model_set_sort_order (RhythmDBQueryModel *model,
 			ptr = g_sequence_ptr_next (ptr);
 		}
 
-		/* generate resort map and rebuild reverse map */
-		reorder_map = malloc (length * sizeof(gint));
-
-		ptr = g_sequence_get_begin_ptr (new_entries);
-		for (i = 0; i < length; i++) {
-			gpointer entry = g_sequence_ptr_get_data (ptr);
-			GSequencePtr old_ptr;
-		       
-			old_ptr = g_hash_table_lookup (model->priv->reverse_map, entry);
-			reorder_map[i] = g_sequence_ptr_get_position (ptr);
-			g_hash_table_replace (model->priv->reverse_map, entry, ptr);
-
-			ptr = g_sequence_ptr_next (ptr);
-		}
-
-		/* emit the re-order and clean up */
-		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter);
-		path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
-		gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model),
-					       path, &iter,
-					       reorder_map);
-
-		gtk_tree_path_free (path);
-		free (reorder_map);
-		g_sequence_free (model->priv->entries);
-		model->priv->entries = new_entries;
+		apply_updated_entry_sequence (model, new_entries);
 	}
 }
 
@@ -1860,7 +1925,7 @@ rhythmdb_query_model_child_index_to_base_index (RhythmDBQueryModel *model, int i
 	return g_sequence_ptr_get_position (ptr);
 }
 
-static int
+/*static int
 rhythmdb_query_model_base_index_to_child_index (RhythmDBQueryModel *model, int index)
 {
 	GSequencePtr ptr;
@@ -1881,9 +1946,8 @@ rhythmdb_query_model_base_index_to_child_index (RhythmDBQueryModel *model, int i
 		return -1;
 
 	pos = g_sequence_ptr_get_position (ptr);
-	g_assert (index == rhythmdb_query_model_child_index_to_base_index (model, pos));
 	return pos;
-}
+}*/
 
 static int
 rhythmdb_query_model_get_entry_index (RhythmDBQueryModel *model, RhythmDBEntry *entry)
@@ -1960,20 +2024,6 @@ rhythmdb_query_model_base_complete (GtkTreeModel *base_model,
 	g_signal_emit (G_OBJECT (model), rhythmdb_query_model_signals[COMPLETE], 0);
 }
 
-static void
-rhythmdb_query_model_base_entry_prop_changed (GtkTreeModel *base_model,
-					      RhythmDBEntry *entry,
-					      RhythmDBPropType prop,
-					      const GValue *old,
-					      const GValue *new,
-					      RhythmDBQueryModel *model)
-{
-	if (g_hash_table_lookup (model->priv->reverse_map, entry)) {
-		g_signal_emit (G_OBJECT (model), rhythmdb_query_model_signals[ENTRY_PROP_CHANGED], 0,
-			       entry, prop, old, new);
-	}
-}
-
 static void
 rhythmdb_query_model_base_rows_reordered (GtkTreeModel *base_model,
 					  GtkTreePath *arg1,
@@ -1981,32 +2031,42 @@ rhythmdb_query_model_base_rows_reordered (GtkTreeModel *base_model,
 					  gint *order_map,
 					  RhythmDBQueryModel *model)
 {
-	gint new_length;
-	gint *new_map;
-	gint old_length;
-	gint i;
-	GtkTreePath *path;
-	GtkTreeIter iter;
+	RhythmDBQueryModel *base_query_model = RHYTHMDB_QUERY_MODEL (base_model);
+	GSequence *new_entries;
+	GSequencePtr ptr;
 
-	old_length = g_sequence_get_length (RHYTHMDB_QUERY_MODEL (base_model)->priv->entries);
-	new_length = g_sequence_get_length (model->priv->entries);
-	new_map = malloc (new_length * sizeof(gint));
+	/* ignore, if this model sorts */
+	if (model->priv->sort_func)
+		return;
 
-	/* convert the reorder map from the base to child models */
-	for (i = 0; i < new_length; i++) {
-		gint old_index = rhythmdb_query_model_child_index_to_base_index (model, i);
-		gint new_index = rhythmdb_query_model_base_index_to_child_index (model, order_map[old_index]);
+	new_entries = g_sequence_new (NULL);
 
-		new_map[i] = new_index;
+	/* iterate over the entries in the base, and recreate the sequence */
+	ptr = g_sequence_get_begin_ptr (base_query_model->priv->entries);
+	while (!g_sequence_ptr_is_end (ptr)) {
+		gpointer entry = g_sequence_ptr_get_data (ptr);
+
+		if (g_hash_table_lookup (model->priv->reverse_map, entry))
+			g_sequence_append (new_entries, entry);
+
+		ptr = g_sequence_ptr_next (ptr);
 	}
 
-	gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter);
-	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
-	gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, NULL, new_map);
-	gtk_tree_path_free (path);
-	free (new_map);
+	apply_updated_entry_sequence (model, new_entries);
 }
 
+static void
+rhythmdb_query_model_base_entry_removed (RhythmDBQueryModel *base_model,
+					 RhythmDBEntry *entry,
+					 RhythmDBQueryModel *model)
+{
+	/* propagate the signal out to any attached property models */
+	g_signal_emit (G_OBJECT (model),
+		       rhythmdb_query_model_signals[ENTRY_REMOVED], 0,
+		       entry);
+}
+
+
 void
 rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter)
 {
@@ -2014,6 +2074,7 @@ rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter)
 	GSequencePtr next;
 	RhythmDBEntry *entry;
 	gboolean changed = FALSE;
+	GList *remove = NULL;
 
 	/* process limited list first, so entries that don't match can't sneak in 
 	 * to the main list from there
@@ -2026,12 +2087,23 @@ rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter)
 			if (!rhythmdb_evaluate_query (model->priv->db, 
 						      model->priv->query, 
 						      entry)) {
-				rhythmdb_query_model_remove_from_limited_list (model, entry);
-				changed = TRUE;
+				remove = g_list_prepend (remove, entry);
 			}
 
 			ptr = next;
 		}
+
+		if (remove) {
+			GList *t;
+			for (t = remove; t; t = t->next) {
+				RhythmDBEntry *entry = t->data;
+				rhythmdb_query_model_remove_from_limited_list (model, entry);
+			}
+			
+			changed = TRUE;
+			g_list_free (remove);
+			remove = NULL;
+		}
 	}
 
 	if (model->priv->entries) {
@@ -2042,16 +2114,27 @@ rhythmdb_query_model_reapply_query (RhythmDBQueryModel *model, gboolean filter)
 			if (!rhythmdb_evaluate_query (model->priv->db, 
 						      model->priv->query, 
 						      entry)) {
+				remove = g_list_prepend (remove, entry);
+			}
+
+			ptr = next;
+		}
+
+		if (remove) {
+			GList *t;
+			for (t = remove; t; t = t->next) {
+				RhythmDBEntry *entry = t->data;
 				if (!filter) {
 					g_signal_emit (G_OBJECT (model),
 						       rhythmdb_query_model_signals[ENTRY_REMOVED], 0,
 						       entry);
 				}
 				rhythmdb_query_model_remove_from_main_list (model, entry);
-				changed = TRUE;
 			}
-
-			ptr = next;
+			
+			changed = TRUE;
+			g_list_free (remove);
+			remove = NULL;
 		}
 	}
 
@@ -2087,12 +2170,42 @@ rhythmdb_query_model_location_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 		return strcmp (a_val, b_val);
 }
 
+gint
+rhythmdb_query_model_title_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+				      RhythmDBQueryModel *model)
+{
+	const char *a_val;
+	const char *b_val;
+	gint ret;
+
+	/* Sort by album name */
+	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE_SORT_KEY);
+	b_val = rhythmdb_entry_get_string (b, RHYTHMDB_PROP_TITLE_SORT_KEY);
+
+	if (a_val == NULL) {
+		if (b_val == NULL)
+			ret = 0;
+		else
+			ret = -1;
+	} else if (b_val == NULL)
+		ret = 1;
+	else
+		ret = strcmp (a_val, b_val);
+
+	if (ret != 0)
+		return ret;
+	else
+		return rhythmdb_query_model_location_sort_func (a, b, model);
+}
+
 gint
 rhythmdb_query_model_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				      RhythmDBQueryModel *model)
 {
 	const char *a_val;
 	const char *b_val;
+	gulong a_num;
+	gulong b_num;
 	gint ret;
 
 	/* Sort by album name */
@@ -2113,12 +2226,16 @@ rhythmdb_query_model_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 		return ret;
 
 	/* Then by disc number, */
-	if (a->discnum != b->discnum)
-		return (a->discnum < b->discnum ? -1 : 1);
+	a_num = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_DISC_NUMBER);
+	b_num = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_DISC_NUMBER);
+	if (a_num != b_num)
+		return (a_num < b_num ? -1 : 1);
 
 	/* by track number */
-	if (a->tracknum != b->tracknum)
-		return (a->tracknum < b->tracknum ? -1 : 1);
+	a_num = rhythmdb_entry_get_ulong (a, RHYTHMDB_PROP_TRACK_NUMBER);
+	b_num = rhythmdb_entry_get_ulong (b, RHYTHMDB_PROP_TRACK_NUMBER);
+	if (a_num != b_num)
+		return (a_num < b_num ? -1 : 1);
 
 	/*  by title */
 	a_val = rhythmdb_entry_get_string (a, RHYTHMDB_PROP_TITLE_SORT_KEY);
