@@ -28,17 +28,31 @@
 #include <string.h>
 
 #include "rb-removable-media-manager.h"
+#include "rb-library-source.h"
 #include "rb-sourcelist.h"
 #include "rb-removable-media-source.h"
 #include "rb-generic-player-source.h"
 #include "rb-audiocd-source.h"
+#include "rb-psp-source.h"
+#ifdef HAVE_HAL
+#include "rb-nokia770-source.h"
+#endif
 #ifdef WITH_IPOD_SUPPORT
 #include "rb-ipod-source.h"
 #endif
 
+#include "rb-shell.h"
+#include "rb-shell-player.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
+#include "rb-stock-icons.h"
 #include "rhythmdb.h"
+#include "rb-marshal.h"
+#include "rb-util.h"
+
+#ifdef ENABLE_TRACK_TRANSFER
+#include "rb-encoder.h"
+#endif
 
 #ifndef HAVE_BURN_DRIVE_UNREF
 #define nautilus_burn_drive_unref nautilus_burn_drive_free
@@ -47,6 +61,7 @@
 static void rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass);
 static void rb_removable_media_manager_init (RBRemovableMediaManager *mgr);
 static void rb_removable_media_manager_dispose (GObject *object);
+static void rb_removable_media_manager_finalize (GObject *object);
 static void rb_removable_media_manager_set_property (GObject *object,
 					      guint prop_id,
 					      const GValue *value,
@@ -78,6 +93,12 @@ static void  rb_removable_media_manager_volume_unmounted_cb (GnomeVFSVolumeMonit
 				gpointer data);
 static void rb_removable_media_manager_scan (RBRemovableMediaManager *manager);
 
+#ifdef ENABLE_TRACK_TRANSFER
+static void do_transfer (RBRemovableMediaManager *manager);
+#endif
+static void rb_removable_media_manager_cmd_copy_tracks (GtkAction *action,
+							RBRemovableMediaManager *mgr);
+
 typedef struct
 {
 	gboolean removed;
@@ -101,6 +122,14 @@ typedef struct
 	GHashTable *volume_mapping;
 	GHashTable *cd_drive_mapping;
 	GList *cur_volume_list;
+
+	char *playing_uri;
+
+	GAsyncQueue *transfer_queue;
+	gboolean transfer_running;
+	gint transfer_total;
+	gint transfer_done;
+	double transfer_fraction;
 } RBRemovableMediaManagerPrivate;
 
 G_DEFINE_TYPE (RBRemovableMediaManager, rb_removable_media_manager, G_TYPE_OBJECT)
@@ -117,6 +146,7 @@ enum
 enum
 {
 	MEDIUM_ADDED,
+	TRANSFER_PROGRESS,
 	LAST_SIGNAL
 };
 
@@ -124,9 +154,12 @@ static guint rb_removable_media_manager_signals[LAST_SIGNAL] = { 0 };
 
 static GtkActionEntry rb_removable_media_manager_actions [] =
 {
-	{ "RemovableSourceEject", NULL, N_("_Eject"), NULL,
+	{ "RemovableSourceEject", GNOME_MEDIA_EJECT, N_("_Eject"), NULL,
 	  N_("Eject this medium"),
 	  G_CALLBACK (rb_removable_media_manager_cmd_eject_medium) },
+	{ "RemovableSourceCopyAllTracks", GTK_STOCK_CDROM, N_("_Copy to library"), NULL,
+	  N_("Copy all tracks to the library"),
+	  G_CALLBACK (rb_removable_media_manager_cmd_copy_tracks) },
 	{ "MusicScanMedia", NULL, N_("_Scan Removable Media"), NULL,
 	  N_("Scan for new Removable Media"),
 	  G_CALLBACK (rb_removable_media_manager_cmd_scan_media) },
@@ -140,7 +173,7 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->dispose = rb_removable_media_manager_dispose;
-
+	object_class->finalize = rb_removable_media_manager_finalize;
 	object_class->set_property = rb_removable_media_manager_set_property;
 	object_class->get_property = rb_removable_media_manager_get_property;
 
@@ -177,6 +210,17 @@ rb_removable_media_manager_class_init (RBRemovableMediaManagerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_OBJECT);
 
+	rb_removable_media_manager_signals[TRANSFER_PROGRESS] =
+		g_signal_new ("transfer-progress",
+			      RB_TYPE_REMOVABLE_MEDIA_MANAGER,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBRemovableMediaManagerClass, transfer_progress),
+			      NULL, NULL,
+			      rb_marshal_VOID__INT_INT_DOUBLE,
+			      G_TYPE_NONE,
+			      3, G_TYPE_INT, G_TYPE_INT, G_TYPE_DOUBLE);
+
+
 	g_type_class_add_private (klass, sizeof (RBRemovableMediaManagerPrivate));
 }
 
@@ -186,6 +230,7 @@ rb_removable_media_manager_init (RBRemovableMediaManager *mgr)
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
 
 	priv->volume_mapping = g_hash_table_new (NULL, NULL);
+	priv->transfer_queue = g_async_queue_new ();
 }
 
 static void
@@ -210,20 +255,24 @@ rb_removable_media_manager_dispose (GObject *object)
 		g_list_free (priv->sources);
 		priv->sources = NULL;
 	}
-	if (priv->volume_mapping) {
-		g_hash_table_destroy (priv->volume_mapping);
-		priv->volume_mapping = NULL;
-	}
-	if (priv->cd_drive_mapping) {
-		g_hash_table_destroy (priv->cd_drive_mapping);
-		priv->cd_drive_mapping = NULL;
-	}
 
 	priv->disposed = TRUE;
 
 	G_OBJECT_CLASS (rb_removable_media_manager_parent_class)->dispose (object);
 }
 
+static void
+rb_removable_media_manager_finalize (GObject *object)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (object);
+
+	g_hash_table_destroy (priv->volume_mapping);
+	g_hash_table_destroy (priv->cd_drive_mapping);
+	g_async_queue_unref (priv->transfer_queue);
+	
+	G_OBJECT_CLASS (rb_removable_media_manager_parent_class)->finalize (object);
+}
+
 static void
 rb_removable_media_manager_set_property (GObject *object,
 				  guint prop_id,
@@ -345,22 +394,126 @@ void begin_cd_drive_monitor (NautilusBurnDrive *drive, RBRemovableMediaManager *
 	info->drive = drive;
 	info->tray_opened = nautilus_burn_drive_door_is_open (drive);
 	info->manager = manager;
+
 	g_hash_table_insert (priv->cd_drive_mapping, drive, info);
 	g_timeout_add (1000, (GSourceFunc)poll_tray_opened, info);
-	if (!nautilus_burn_drive_door_is_open (drive)) {
-		volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, drive->device);
-		if (volume)
+
+	volume = gnome_vfs_volume_monitor_get_volume_for_path (monitor, drive->device);
+
+	if (volume) {
+		if (!nautilus_burn_drive_door_is_open (drive)) {
 			rb_removable_media_manager_mount_volume (manager, volume);
+		} else {
+			/* it may have got ejected while we weren't monitoring */
+			rb_removable_media_manager_unmount_volume (manager, volume);
+		}
 	}
 #endif
 }
 
+static char *
+split_drive_from_cdda_uri (const char *uri)
+{
+	gchar *copy, *temp, *split;
+	int len;
+
+	if (!g_str_has_prefix (uri, "cdda://"))
+		return NULL;
+
+	len = strlen ("cdda://");
+	
+	copy = g_strdup (uri);
+	split = g_utf8_strrchr (copy + len, -1, ':');
+
+	if (split == NULL) {
+		/* invalid URI, it doesn't contain a ':' */
+		g_free (copy);
+		return NULL;
+	}
+
+	*split = 0;
+	temp = g_strdup (copy + len);
+	g_free (copy);
+
+	return temp;
+}
+
+static NautilusBurnDrive *
+get_nautilus_burn_drive_for_path (const char *path)
+{
+#ifdef HAVE_BURN_DRIVE_NEW_FROM_PATH
+	return nautilus_burn_drive_new_from_path (path);
+#else
+	GList *drives, *l;
+	NautilusBurnDrive *path_drive = NULL;
+
+	drives = nautilus_burn_drive_get_list (FALSE, FALSE);
+	for (l = drives; l != NULL; l = g_list_next (l)) {
+		NautilusBurnDrive *drive = (NautilusBurnDrive*)l->data;
+
+		if (path_drive == NULL && strcmp (drive->device, path) == 0) {
+			path_drive = drive;
+		} else {
+			nautilus_burn_drive_unref (drive);
+		}
+	}
+	g_list_free (drives);
+
+	return path_drive;
+#endif
+}
+
+static void
+rb_removable_media_manager_playing_uri_changed_cb (RBShellPlayer *player,
+						   const char *uri,
+						   RBRemovableMediaManager *manager)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
+	char *old_drive = NULL;
+	char *new_drive = NULL;
+
+	/* extract the drive paths */
+	if (priv->playing_uri)
+		old_drive = split_drive_from_cdda_uri (priv->playing_uri);
+
+	if (uri)
+		new_drive = split_drive_from_cdda_uri (uri);
+
+
+	/* if the drive we're playing from has changed, adjust the polling */
+	if (old_drive == NULL || new_drive == NULL || strcmp (old_drive, new_drive) != 0) {
+		if (old_drive) {
+			NautilusBurnDrive *drive;
+
+			rb_debug ("restarting monitoring of drive %s after playing", old_drive);
+			drive = get_nautilus_burn_drive_for_path (old_drive);
+			begin_cd_drive_monitor (drive, manager);
+			nautilus_burn_drive_unref (drive);
+		}
+		
+		if (new_drive) {
+			NautilusBurnDrive *drive;
+
+			rb_debug ("stopping monitoring of drive %s while playing", new_drive);
+			drive = get_nautilus_burn_drive_for_path (new_drive);
+			/* removing it from the hash table makes it stop monitoring */
+			g_hash_table_remove (priv->cd_drive_mapping, drive);
+			nautilus_burn_drive_unref (drive);
+		}
+	}
+
+	g_free (priv->playing_uri);
+	priv->playing_uri = (uri) ? g_strdup (uri) : NULL;
+}
+
+	
 gboolean
 rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 {
 	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
 	GnomeVFSVolumeMonitor *monitor = gnome_vfs_get_volume_monitor ();
 	GList *drives;
+	GObject *shell_player;
 
 	/*
 	 * Monitor new (un)mounted file systems to look for new media
@@ -391,6 +544,12 @@ rb_removable_media_manager_load_media (RBRemovableMediaManager *manager)
 	g_list_foreach (drives, (GFunc)begin_cd_drive_monitor, manager);
 	g_list_free (drives);
 
+	/* monitor the playing song, to disable cd drive polling */
+	shell_player = rb_shell_get_player (priv->shell);
+	g_signal_connect (shell_player, "playing-uri-changed",
+			  G_CALLBACK (rb_removable_media_manager_playing_uri_changed_cb),
+			  manager);
+
 	/* scan for media */
 	rb_removable_media_manager_scan (manager);
 
@@ -452,8 +611,8 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 	display_name = gnome_vfs_volume_get_display_name (volume);
 	hal_udi = gnome_vfs_volume_get_hal_udi (volume);
 	icon_name = gnome_vfs_volume_get_icon (volume);
-	rb_debug ("detecting new media - device_type=%d", device_type);
-	rb_debug ("detecting new media - volumd_type=%d", gnome_vfs_volume_get_volume_type (volume));
+	rb_debug ("detecting new media - device type=%d", device_type);
+	rb_debug ("detecting new media - volume type=%d", gnome_vfs_volume_get_volume_type (volume));
 	rb_debug ("detecting new media - fs type=%s", fs_type);
 	rb_debug ("detecting new media - device path=%s", device_path);
 	rb_debug ("detecting new media - display name=%s", display_name);
@@ -471,6 +630,12 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 #ifdef WITH_IPOD_SUPPORT
 	if (source == NULL && rb_ipod_is_volume_ipod (volume))
 		source = rb_ipod_source_new (shell, volume);
+#endif
+	if (source == NULL && rb_psp_is_volume_player (volume))
+		source = rb_psp_source_new (shell, volume);
+#ifdef HAVE_HAL
+	if (source == NULL && rb_nokia770_is_volume_player (volume))
+		source = rb_nokia770_source_new (shell, volume);
 #endif
 	if (source == NULL && rb_generic_player_is_volume_player (volume))
 		source = rb_generic_player_source_new (shell, volume);
@@ -479,7 +644,7 @@ rb_removable_media_manager_mount_volume (RBRemovableMediaManager *mgr, GnomeVFSV
 		g_hash_table_insert (priv->volume_mapping, volume, source);
 		rb_removable_media_manager_append_media_source (mgr, source);
 	} else
-		rb_debug ("Unhanded media");
+		rb_debug ("Unhandled media");
 
 	g_free (fs_type);
 	g_free (device_path);
@@ -655,3 +820,169 @@ rb_removable_media_manager_scan (RBRemovableMediaManager *manager)
 	}
 	g_list_free (list);
 }
+
+
+#ifdef ENABLE_TRACK_TRANSFER
+/* Track transfer */
+
+typedef struct {
+	RBRemovableMediaManager *manager;
+	RhythmDBEntry *entry;
+	char *dest;
+	char *mime_type;
+	gboolean failed;
+	RBTranferCompleteCallback callback;
+	gpointer userdata;
+} TransferData;
+
+static void
+emit_progress (RBRemovableMediaManager *mgr)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
+
+	g_signal_emit (G_OBJECT (mgr), rb_removable_media_manager_signals[TRANSFER_PROGRESS], 0,
+		       priv->transfer_done,
+		       priv->transfer_total,
+		       priv->transfer_fraction);
+}
+
+static void
+error_cb (RBEncoder *encoder, GError *error, TransferData *data)
+{
+	rb_debug ("Error transferring track to %s: %s", data->dest, error->message);
+	rb_error_dialog (NULL, _("Error transferring track"), "%s", error->message);
+
+	data->failed = TRUE;
+	rb_encoder_cancel (encoder);
+}
+
+static void
+progress_cb (RBEncoder *encoder, double fraction, TransferData *data)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (data->manager);
+
+	rb_debug ("transfer progress %f", (float)fraction);
+	priv->transfer_fraction = fraction;
+	emit_progress (data->manager);
+}
+
+static void
+completed_cb (RBEncoder *encoder, TransferData *data)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (data->manager);
+
+	rb_debug ("completed transferring track to %s", data->dest);
+	if (!data->failed)
+		(data->callback) (data->entry, data->dest, data->userdata);
+
+	priv->transfer_running = FALSE;
+	priv->transfer_done++;
+	priv->transfer_fraction = 0.0;
+	do_transfer (data->manager);
+
+	g_object_unref (G_OBJECT (encoder));
+	g_free (data->dest);
+	g_free (data->mime_type);
+	g_free (data);
+}
+
+static void
+do_transfer (RBRemovableMediaManager *manager)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
+	TransferData *data;
+	RBEncoder *encoder;
+
+	g_assert (rb_is_main_thread ());
+	
+	emit_progress (manager);
+
+	if (priv->transfer_running)
+		return;
+
+	data = g_async_queue_try_pop (priv->transfer_queue);
+	if (data == NULL) {
+		priv->transfer_total = 0;
+		priv->transfer_done = 0;
+		emit_progress (manager);
+		return;
+	}
+
+	priv->transfer_running = TRUE;
+	priv->transfer_fraction = 0.0;
+
+	encoder = rb_encoder_new ();
+	g_signal_connect (G_OBJECT (encoder),
+			  "error", G_CALLBACK (error_cb),
+			  data);
+	g_signal_connect (G_OBJECT (encoder),
+			  "progress", G_CALLBACK (progress_cb),
+			  data);
+	g_signal_connect (G_OBJECT (encoder),
+			  "completed", G_CALLBACK (completed_cb),
+			  data);
+	rb_encoder_encode (encoder, data->entry, data->dest, NULL);
+}
+
+void
+rb_removable_media_manager_queue_transfer (RBRemovableMediaManager *manager,
+					  RhythmDBEntry *entry,
+					  const char *dest,
+					  const char *mime_type,
+					  RBTranferCompleteCallback callback,
+					  gpointer userdata)
+{
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (manager);
+	TransferData *data;
+
+	g_assert (rb_is_main_thread ());
+
+	data = g_new0 (TransferData, 1);
+	data->manager = manager;
+	data->entry = entry;
+	data->dest = g_strdup (dest);
+	data->mime_type = g_strdup (mime_type);
+	data->callback = callback;
+	data->userdata = userdata;
+
+	g_async_queue_push (priv->transfer_queue, data);
+	priv->transfer_total++;
+	do_transfer (manager);
+}
+
+static gboolean
+copy_entry (RhythmDBQueryModel *model,
+	    GtkTreePath *path,
+	    GtkTreeIter *iter,
+	    GList **list)
+{
+	GList *l;
+	l = g_list_append (*list, rhythmdb_query_model_iter_to_entry (model, iter));
+	*list = l;
+	return FALSE;
+}
+#endif
+
+static void
+rb_removable_media_manager_cmd_copy_tracks (GtkAction *action, RBRemovableMediaManager *mgr)
+{
+#ifdef ENABLE_TRACK_TRANSFER
+	RBRemovableMediaManagerPrivate *priv = REMOVABLE_MEDIA_MANAGER_GET_PRIVATE (mgr);
+	RBRemovableMediaSource *source;
+	RBLibrarySource *library;
+	RhythmDBQueryModel *model;
+	GList *list = NULL;
+
+	source = RB_REMOVABLE_MEDIA_SOURCE (priv->selected_source);
+	g_object_get (G_OBJECT (source), "query-model", &model, NULL);
+	g_object_get (G_OBJECT (priv->shell), "library-source", &library, NULL);
+
+	gtk_tree_model_foreach (GTK_TREE_MODEL (model), (GtkTreeModelForeachFunc)copy_entry, &list);
+	rb_source_paste (RB_SOURCE (library), list);
+	g_list_free (list);
+
+	g_object_unref (G_OBJECT (model));
+	g_object_unref (G_OBJECT (library));
+#endif
+}
+
