@@ -46,7 +46,7 @@ static void rb_shell_clipboard_get_property (GObject *object,
 				   	     guint prop_id,
 					     GValue *value,
 					     GParamSpec *pspec);
-static void rb_shell_clipboard_sync (RBShellClipboard *clipboard);
+static gboolean rb_shell_clipboard_sync (RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_select_all (GtkAction *action,
 					       RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_select_none (GtkAction *action,
@@ -71,6 +71,9 @@ static void rb_shell_clipboard_entry_deleted_cb (RhythmDB *db,
 						 RBShellClipboard *clipboard);
 static void rb_shell_clipboard_entryview_changed_cb (RBEntryView *view,
 						     RBShellClipboard *clipboard);
+static void rb_shell_clipboard_entries_changed_cb (RBEntryView *view,
+						   gpointer stuff,
+						   RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_add_song_to_queue (GtkAction *action,
 						      RBShellClipboard *clipboard);
 static void rb_shell_clipboard_cmd_song_info (GtkAction *action,
@@ -90,7 +93,7 @@ struct RBShellClipboardPrivate
 
 	GAsyncQueue *deleted_queue;
 
-	guint idle_sync_id;
+	guint idle_deletion_id, idle_sync_id;
 
 	GList *entries;
 };
@@ -126,7 +129,7 @@ static GtkActionEntry rb_shell_clipboard_actions [] =
 	{ "EditDelete", GTK_STOCK_REMOVE, N_("_Remove"), NULL,
 	  N_("Remove selection"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_delete) },
-	{ "EditMovetoTrash", GTK_STOCK_DELETE, N_("_Move To Trash"), "<control>T",
+	{ "EditMovetoTrash", GTK_STOCK_DELETE, N_("_Move to Trash"), "<control>T",
 	  N_("Move selection to the trash"),
 	  G_CALLBACK (rb_shell_clipboard_cmd_move_to_trash) },
 
@@ -201,7 +204,7 @@ rb_shell_clipboard_init (RBShellClipboard *shell_clipboard)
 
 	shell_clipboard->priv->deleted_queue = g_async_queue_new ();
 
-	shell_clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_idle_poll_deletions, shell_clipboard); 
+	shell_clipboard->priv->idle_deletion_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_idle_poll_deletions, shell_clipboard); 
 }
 
 static void
@@ -224,6 +227,8 @@ rb_shell_clipboard_finalize (GObject *object)
 
 	if (shell_clipboard->priv->idle_sync_id)
 		g_source_remove (shell_clipboard->priv->idle_sync_id);
+	if (shell_clipboard->priv->idle_deletion_id)
+		g_source_remove (shell_clipboard->priv->idle_deletion_id);
 	
 	G_OBJECT_CLASS (rb_shell_clipboard_parent_class)->finalize (object);
 }
@@ -251,6 +256,51 @@ rb_shell_clipboard_constructor (GType type, guint n_construct_properties,
 	return G_OBJECT (clip);
 }
 
+static void
+rb_shell_clipboard_set_source_internal (RBShellClipboard *clipboard,
+					RBSource *source)
+{
+	if (clipboard->priv->source != NULL) {
+		RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
+
+		if (songs) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
+							      G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
+							      clipboard);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
+							      G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
+							      clipboard);
+		}
+	}
+	clipboard->priv->source = source;
+	rb_debug ("selected source %p", source);
+
+	rb_shell_clipboard_sync (clipboard);
+
+	if (clipboard->priv->source != NULL) {
+		RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
+
+		if (songs) {
+			g_signal_connect_object (G_OBJECT (songs),
+						 "selection-changed",
+						 G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
+						 clipboard, 0);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "entry-added",
+						 G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
+						 clipboard, 0);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "entry-deleted",
+						 G_CALLBACK (rb_shell_clipboard_entries_changed_cb),
+						 clipboard, 0);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "entries-replaced",
+						 G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
+						 clipboard, 0);
+		}
+	}
+}
+
 static void
 rb_shell_clipboard_set_property (GObject *object,
 			         guint prop_id,
@@ -262,28 +312,7 @@ rb_shell_clipboard_set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_SOURCE:
-		if (clipboard->priv->source != NULL)
-		{
-			RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
-
-			g_signal_handlers_disconnect_by_func (G_OBJECT (songs),
-							      G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
-							      clipboard);
-		}
-		clipboard->priv->source = g_value_get_object (value);
-		rb_debug ("selected source %p", g_value_get_object (value));
-
-		rb_shell_clipboard_sync (clipboard);
-
-		if (clipboard->priv->source != NULL)
-		{
-			RBEntryView *songs = rb_source_get_entry_view (clipboard->priv->source);
-
-			g_signal_connect_object (G_OBJECT (songs),
-						 "selection-changed",
-						 G_CALLBACK (rb_shell_clipboard_entryview_changed_cb),
-						 clipboard, 0);
-		}
+		rb_shell_clipboard_set_source_internal (clipboard, g_value_get_object (value));
 		break;
 	case PROP_ACTION_GROUP:
 		clipboard->priv->actiongroup = g_value_get_object (value);
@@ -360,10 +389,11 @@ rb_shell_clipboard_new (GtkActionGroup *actiongroup, RhythmDB *db)
 			     NULL);
 }
 
-static void
+static gboolean
 rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 {
-	gboolean have_selection;
+	RBEntryView *view;
+	gboolean have_selection = FALSE;
 	gboolean have_sidebar_selection = FALSE;
 	gboolean can_cut = FALSE;
 	gboolean can_paste = FALSE;
@@ -371,12 +401,20 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	gboolean can_copy = FALSE;
 	gboolean can_add_to_queue = FALSE;
 	gboolean can_move_to_trash = FALSE;
+	gboolean can_select_all = FALSE;
 	GtkAction *action;
 
 	if (!clipboard->priv->source)
-		return;
+		return FALSE;
+
+	clipboard->priv->idle_sync_id = 0;
+
+	view = rb_source_get_entry_view (clipboard->priv->source);
+	if (view) {
+		have_selection = rb_entry_view_have_selection (view);
+		can_select_all = !rb_entry_view_have_complete_selection (view);
+	}
 
-	have_selection = rb_entry_view_have_selection (rb_source_get_entry_view (clipboard->priv->source));
 	if (clipboard->priv->queue_source) {
 		RBEntryView *sidebar;
 		g_object_get (G_OBJECT (clipboard->priv->queue_source), "sidebar", &sidebar, NULL);
@@ -387,48 +425,52 @@ rb_shell_clipboard_sync (RBShellClipboard *clipboard)
 	rb_debug ("syncing clipboard");
 	
 	if (g_list_length (clipboard->priv->entries) > 0)
-		can_paste = rb_source_can_cut (clipboard->priv->source);
+		can_paste = rb_source_can_paste (clipboard->priv->source);
 
 	if (have_selection) {
 		can_cut = rb_source_can_cut (clipboard->priv->source);
 		can_delete = rb_source_can_delete (clipboard->priv->source);
 		can_copy = rb_source_can_copy (clipboard->priv->source);
 		can_move_to_trash = rb_source_can_move_to_trash (clipboard->priv->source);
+
 		if (clipboard->priv->queue_source)
 			can_add_to_queue = rb_source_can_add_to_queue (clipboard->priv->source);
 	}
 
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "EditCut");
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCut");
 	g_object_set (G_OBJECT (action), "sensitive", can_cut, NULL);
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "EditDelete");
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditDelete");
 	g_object_set (G_OBJECT (action), "sensitive", can_delete, NULL);
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "EditMovetoTrash");
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditMovetoTrash");
 	g_object_set (G_OBJECT (action), "sensitive", can_move_to_trash, NULL);
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "EditCopy");
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditCopy");
 	g_object_set (G_OBJECT (action), "sensitive", can_copy, NULL);
 
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "EditPaste");
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup,"EditPaste");
 	g_object_set (G_OBJECT (action), "sensitive", can_paste, NULL);
 
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "AddToQueue");
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "AddToQueue");
 	g_object_set (G_OBJECT (action), "sensitive", can_add_to_queue, NULL);
 	
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "MusicProperties");
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "MusicProperties");
 	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
 	
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "QueueMusicProperties");
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueMusicProperties");
 	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
-	action = gtk_action_group_get_action (clipboard->priv->actiongroup,
-					      "QueueDelete");
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "QueueDelete");
 	g_object_set (G_OBJECT (action), "sensitive", have_sidebar_selection, NULL);
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectAll");
+	g_object_set (G_OBJECT (action), "sensitive", can_select_all, NULL);
+	
+	action = gtk_action_group_get_action (clipboard->priv->actiongroup, "EditSelectNone");
+	g_object_set (G_OBJECT (action), "sensitive", have_selection, NULL);
+
+	return FALSE;
 }
 
 static void
@@ -553,12 +595,12 @@ rb_shell_clipboard_idle_poll_deletions (RBShellClipboard *clipboard)
 	did_sync = rb_shell_clipboard_process_deletions (clipboard);
 
 	if (did_sync)
-		clipboard->priv->idle_sync_id =
+		clipboard->priv->idle_deletion_id =
 			g_idle_add_full (G_PRIORITY_LOW,
 					 (GSourceFunc) rb_shell_clipboard_idle_poll_deletions,
 					 clipboard, NULL);
 	else
-		clipboard->priv->idle_sync_id =
+		clipboard->priv->idle_deletion_id =
 			g_timeout_add (300,
 				       (GSourceFunc) rb_shell_clipboard_idle_poll_deletions,
 				       clipboard);
@@ -582,8 +624,21 @@ static void
 rb_shell_clipboard_entryview_changed_cb (RBEntryView *view,
 					 RBShellClipboard *clipboard)
 {
+	if (clipboard->priv->idle_sync_id == 0)
+		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync,
+							    clipboard);
 	rb_debug ("entryview changed");
-	rb_shell_clipboard_sync (clipboard);
+}
+	
+static void
+rb_shell_clipboard_entries_changed_cb (RBEntryView *view,
+				       gpointer stuff,
+				       RBShellClipboard *clipboard)
+{
+	rb_debug ("entryview changed");
+	if (clipboard->priv->idle_sync_id == 0)
+		clipboard->priv->idle_sync_id = g_idle_add ((GSourceFunc) rb_shell_clipboard_sync,
+							    clipboard);
 }
 
 static void
