@@ -56,6 +56,10 @@ static void rb_shell_preferences_sync (RBShellPreferences *shell_preferences);
 
 void rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 						   RBShellPreferences *shell_preferences);
+void rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
+						      RBShellPreferences *shell_preferences);
+static void rb_shell_preferences_toolbar_style_cb (GtkComboBox *box,
+						   RBShellPreferences *preferences);
 
 
 enum
@@ -82,6 +86,10 @@ struct RBShellPreferencesPrivate
 	GtkWidget *quality_check;
 	GtkWidget *year_check;
 
+	GSList *browser_views_group;
+
+	GtkWidget *toolbar_style_menu;
+
 	gboolean loading;
 };
 
@@ -119,7 +127,7 @@ help_cb (GtkWidget *widget,
 static void
 rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 {
-	GtkWidget *help;
+	GtkWidget *tmp;
 	GladeXML *xml;
 
 	shell_preferences->priv = RB_SHELL_PREFERENCES_GET_PRIVATE (shell_preferences);
@@ -136,10 +144,10 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	gtk_dialog_add_button (GTK_DIALOG (shell_preferences),
 			       GTK_STOCK_CLOSE,
 			       GTK_RESPONSE_CLOSE);
-	help = gtk_dialog_add_button (GTK_DIALOG (shell_preferences),
+	tmp = gtk_dialog_add_button (GTK_DIALOG (shell_preferences),
 			              GTK_STOCK_HELP,
 			              GTK_RESPONSE_HELP);
-	g_signal_connect_object (G_OBJECT (help), "clicked",
+	g_signal_connect_object (G_OBJECT (tmp), "clicked",
 				 G_CALLBACK (help_cb), shell_preferences, 0);
 	gtk_dialog_set_default_response (GTK_DIALOG (shell_preferences),
 					 GTK_RESPONSE_CLOSE);
@@ -187,16 +195,33 @@ rb_shell_preferences_init (RBShellPreferences *shell_preferences)
 	shell_preferences->priv->first_seen_check =
 		glade_xml_get_widget (xml, "first_seen_check");
 
+	/* browser options */
+	rb_glade_boldify_label (xml, "browser_views_label");
+
+	tmp = glade_xml_get_widget (xml, "library_browser_views_radio");
+	shell_preferences->priv->browser_views_group =
+		g_slist_reverse (g_slist_copy (gtk_radio_button_get_group 
+					       (GTK_RADIO_BUTTON (tmp))));
+
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
 				  glade_xml_get_widget (xml, "general_vbox"),
 				  gtk_label_new (_("General")));
 
-	g_object_unref (G_OBJECT (xml));
+	/* toolbar button style */
+	rb_glade_boldify_label (xml, "toolbar_style_label");
+	shell_preferences->priv->toolbar_style_menu =
+		glade_xml_get_widget (xml, "toolbar_style_menu");
+	gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (shell_preferences->priv->toolbar_style_menu),
+					      rb_combo_box_hyphen_separator_func,
+					      NULL, NULL);
+	g_signal_connect_object (G_OBJECT (shell_preferences->priv->toolbar_style_menu),
+				 "changed", G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
+				 shell_preferences, 0);
 	
 	eel_gconf_notification_add (CONF_UI_DIR,
 				    (GConfClientNotifyFunc) rb_shell_preferences_ui_pref_changed,
 				    shell_preferences);
-
+	g_object_unref (G_OBJECT (xml));
 	rb_shell_preferences_sync (shell_preferences);
 }
 
@@ -247,69 +272,150 @@ rb_shell_preferences_append_view_page (RBShellPreferences *prefs,
 
 #ifdef WITH_DAAP_SUPPORT
 
-#define CONF_ENABLE_SHARING "/apps/rhythmbox/sharing/enable_sharing"
-#define CONF_NAME "/apps/rhythmbox/sharing/share_name"
-
 static void
-share_check_button_toggled_cb (GtkToggleButton *button, GtkWidget *hbox)
+share_check_button_toggled_cb (GtkToggleButton *button,
+			       GtkWidget *widget)
 {
 	gboolean b;
 
 	b = gtk_toggle_button_get_active (button);
 
-	eel_gconf_set_boolean (CONF_ENABLE_SHARING, b);
+	eel_gconf_set_boolean (CONF_DAAP_ENABLE_SHARING, b);
 
-	gtk_widget_set_sensitive (hbox, b);
+	gtk_widget_set_sensitive (widget, b);
+	
+	return;
+}
+
+static void
+password_check_button_toggled_cb (GtkToggleButton *button,
+				  GtkWidget *widget)
+{
+	gboolean b;
+
+	b = gtk_toggle_button_get_active (button);
+
+	eel_gconf_set_boolean (CONF_DAAP_REQUIRE_PASSWORD, b);
+
+	gtk_widget_set_sensitive (widget, b);
 	
 	return;
 }
 
 static gboolean
-share_name_entry_focus_out_event_cb (GtkEntry *entry, GdkEventFocus *event, gpointer data)
+share_name_entry_focus_out_event_cb (GtkEntry *entry,
+				     GdkEventFocus *event,
+				     gpointer data)
 {
-	const gchar *name;
-	char *old_name;
+	gboolean    changed;
+	const char *name;
+	char       *old_name;
 
 	name = gtk_entry_get_text (entry);
-	old_name = eel_gconf_get_string (CONF_NAME);
+	old_name = eel_gconf_get_string (CONF_DAAP_SHARE_NAME);
 
-	if (strcmp (name, old_name) != 0)
-		eel_gconf_set_string (CONF_NAME, name);
+	if (name == NULL && old_name == NULL) {
+		changed = FALSE;
+	} else if (name == NULL || old_name == NULL) {
+		changed = TRUE;
+	} else if (strcmp (name, old_name) != 0) {
+		changed = TRUE;
+	} else {
+		changed = FALSE;
+	}
+
+	if (changed) {
+		eel_gconf_set_string (CONF_DAAP_SHARE_NAME, name);
+	}
 
 	g_free (old_name);
 
 	return FALSE;
 }
 
+static gboolean
+share_password_entry_focus_out_event_cb (GtkEntry *entry,
+					 GdkEventFocus *event,
+					 gpointer data)
+{
+	gboolean    changed;
+	const char *pw;
+	char       *old_pw;
+
+	pw = gtk_entry_get_text (entry);
+	old_pw = eel_gconf_get_string (CONF_DAAP_SHARE_PASSWORD);
+
+	if (pw == NULL && old_pw == NULL) {
+		changed = FALSE;
+	} else if (pw == NULL || old_pw == NULL) {
+		changed = TRUE;
+	} else if (strcmp (pw, old_pw) != 0) {
+		changed = TRUE;
+	} else {
+		changed = FALSE;
+	}
+
+	if (changed) {
+		eel_gconf_set_string (CONF_DAAP_SHARE_PASSWORD, pw);
+	}
+
+	g_free (old_pw);
+
+	return FALSE;
+}
+
 static void
 add_daap_preferences (RBShellPreferences *shell_preferences)
 {
 	GladeXML *xml;
 	GtkWidget *check;
-	GtkWidget *entry;
-	GtkWidget *hbox;
-	gboolean b;
+	GtkWidget *name_entry;
+	GtkWidget *password_entry;
+	GtkWidget *password_check;
+	GtkWidget *box;
+	gboolean sharing_enabled;
+	gboolean require_password;
 
-	gchar *name;
+	char *name;
+	char *password;
 
 	xml = rb_glade_xml_new ("daap-prefs.glade",
 				"daap_vbox",
 				shell_preferences);
 
 	check = glade_xml_get_widget (xml, "daap_enable_check");
-	entry = glade_xml_get_widget (xml, "daap_name_entry");
-	hbox = glade_xml_get_widget (xml, "daap_hbox");
+	password_check = glade_xml_get_widget (xml, "daap_password_check");
+	name_entry = glade_xml_get_widget (xml, "daap_name_entry");
+	password_entry = glade_xml_get_widget (xml, "daap_password_entry");
+	box = glade_xml_get_widget (xml, "daap_box");
 	
-	b = eel_gconf_get_boolean (CONF_ENABLE_SHARING);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), b);
-	g_signal_connect (G_OBJECT (check), "toggled", G_CALLBACK (share_check_button_toggled_cb), hbox);
-	
-	name = eel_gconf_get_string (CONF_NAME);
-	gtk_entry_set_text (GTK_ENTRY (entry), name);
-	g_signal_connect (G_OBJECT (entry), "focus-out-event", G_CALLBACK (share_name_entry_focus_out_event_cb), NULL);	
+	sharing_enabled = eel_gconf_get_boolean (CONF_DAAP_ENABLE_SHARING);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), sharing_enabled);
+	g_signal_connect (check, "toggled", G_CALLBACK (share_check_button_toggled_cb), box);
 
-	gtk_widget_set_sensitive (hbox, b);
+	require_password = eel_gconf_get_boolean (CONF_DAAP_REQUIRE_PASSWORD);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (password_check), require_password);
+	g_signal_connect (password_check, "toggled", G_CALLBACK (password_check_button_toggled_cb), password_entry);
 	
+	name = eel_gconf_get_string (CONF_DAAP_SHARE_NAME);
+	if (name != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (name_entry), name);
+	}
+	g_free (name);
+	g_signal_connect (name_entry, "focus-out-event",
+			  G_CALLBACK (share_name_entry_focus_out_event_cb), NULL);	
+
+	password = eel_gconf_get_string (CONF_DAAP_SHARE_PASSWORD);
+	if (password != NULL) {
+		gtk_entry_set_text (GTK_ENTRY (password_entry), password);
+	}
+	g_free (password);
+	g_signal_connect (password_entry, "focus-out-event",
+			  G_CALLBACK (share_password_entry_focus_out_event_cb), NULL);	
+
+	gtk_widget_set_sensitive (box, sharing_enabled);
+	gtk_widget_set_sensitive (password_entry, require_password);
+
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell_preferences->priv->notebook),
 				  glade_xml_get_widget (xml, "daap_vbox"),
 				  gtk_label_new (_("Sharing")));
@@ -402,8 +508,7 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_LAST_PLAYED";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->year_check))
 		colname = "RHYTHMDB_PROP_DATE";
-	else if (shell_preferences->priv->quality_check
-		 && butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
+	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
 		colname = "RHYTHMDB_PROP_BITRATE";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->first_seen_check))
 		colname = "RHYTHMDB_PROP_FIRST_SEEN";
@@ -453,6 +558,8 @@ static void
 rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 {
 	char *columns;
+	GSList  *l;
+	gint view, i;
 
 	shell_preferences->priv->loading = TRUE;
 
@@ -491,13 +598,60 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 		rb_shell_preferences_sync_column_button (shell_preferences,
 			       				 shell_preferences->priv->first_seen_check,
 							 columns, "RHYTHMDB_PROP_FIRST_SEEN");
-		if (shell_preferences->priv->quality_check)
-			rb_shell_preferences_sync_column_button (shell_preferences,
-				       				 shell_preferences->priv->quality_check,
-								 columns, "RHYTHMDB_PROP_BITRATE");
+		rb_shell_preferences_sync_column_button (shell_preferences,
+			       				 shell_preferences->priv->quality_check,
+							 columns, "RHYTHMDB_PROP_BITRATE");
 	}
 
 	g_free (columns);
 
+	view = eel_gconf_get_integer (CONF_UI_BROWSER_VIEWS);
+	for (l = shell_preferences->priv->browser_views_group, i = 0; l != NULL; l = g_slist_next (l), i++)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (l->data), (i == view));
+
+	/* toolbar style */
+	g_signal_handlers_block_by_func (G_OBJECT (shell_preferences->priv->toolbar_style_menu),
+					 G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
+					 shell_preferences);
+
+	view = eel_gconf_get_integer (CONF_UI_TOOLBAR_STYLE);
+	/* skip the separator row */
+	if (view >= 1)
+		view++;
+	gtk_combo_box_set_active (GTK_COMBO_BOX (shell_preferences->priv->toolbar_style_menu), view);
+
+	g_signal_handlers_unblock_by_func (G_OBJECT (shell_preferences->priv->toolbar_style_menu),
+					   G_CALLBACK (rb_shell_preferences_toolbar_style_cb),
+					   shell_preferences);
+
 	shell_preferences->priv->loading = FALSE;
 }
+
+static void
+rb_shell_preferences_toolbar_style_cb (GtkComboBox *box, RBShellPreferences *preferences)
+{
+	int selection;
+
+	selection = gtk_combo_box_get_active (box);
+
+	/* skip the separator row */
+	if (selection >= 1)
+		selection--;
+
+	eel_gconf_set_integer (CONF_UI_TOOLBAR_STYLE, selection);
+}
+
+void
+rb_shell_preferences_browser_views_activated_cb (GtkWidget *widget,
+						 RBShellPreferences *shell_preferences)
+{
+	int index;
+
+	if (shell_preferences->priv->loading)
+		return;
+
+	index = g_slist_index (shell_preferences->priv->browser_views_group, widget);
+
+	eel_gconf_set_integer (CONF_UI_BROWSER_VIEWS, index);
+}
+
