@@ -35,13 +35,13 @@ G_BEGIN_DECLS
 #define RB_IS_SHELL_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_SHELL))
 #define RB_SHELL_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_SHELL, RBShellClass))
 
-typedef enum
+enum
 {
 	RB_SHELL_ERROR_NO_SUCH_URI,
 	RB_SHELL_ERROR_NO_SUCH_PROPERTY,
 	RB_SHELL_ERROR_IMMUTABLE_PROPERTY,
 	RB_SHELL_ERROR_INVALID_PROPERTY_TYPE
-} RBShellError;
+};
 
 #define RB_SHELL_ERROR rb_shell_error_quark ()
 
@@ -83,8 +83,10 @@ gboolean        rb_shell_add_uri        (RBShell *shell,
 gboolean        rb_shell_load_uri       (RBShell *shell, const char *uri, gboolean play, GError **error);
 
 GObject *       rb_shell_get_player     (RBShell *shell);
-
 const char *    rb_shell_get_player_path(RBShell *shell);
+GObject *	rb_shell_get_playlist_manager (RBShell *shell);
+const char *	rb_shell_get_playlist_manager_path (RBShell *shell);
+GObject *	rb_shell_get_ui_manager (RBShell *shell);
 
 void            rb_shell_toggle_visibility (RBShell *shell);
 
@@ -99,17 +101,27 @@ gboolean        rb_shell_set_song_property (RBShell *shell,
 					    const GValue *value,
 					    GError **error);
 
+gboolean	rb_shell_add_to_queue (RBShell *shell,
+				       const gchar *uri,
+				       GError **error);
+
+gboolean	rb_shell_remove_from_queue (RBShell *shell,
+					    const gchar *uri,
+					    GError **error);
+
 void            rb_shell_hidden_notify  (RBShell *shell,
 					 guint timeout,
 					 const char *primary,
 					 GtkWidget *icon,
 					 const char *secondary);
 
-void		rb_shell_construct	(RBShell *shell);
-
 void            rb_shell_register_entry_type_for_source (RBShell *shell,
 							 RBSource *source,
 							 RhythmDBEntryType type);
+RBSource * rb_shell_get_source_by_entry_type (RBShell *shell,
+					      RhythmDBEntryType type);
+
+gboolean        rb_shell_get_party_mode (RBShell *shell);
 
 void rb_shell_append_source (RBShell *shell, RBSource *source, RBSource *parent);
 
