@@ -52,7 +52,8 @@ typedef struct
 
 GType			rb_source_header_get_type	(void);
 
-RBSourceHeader *	rb_source_header_new		(GtkActionGroup *actiongroup);
+RBSourceHeader *	rb_source_header_new		(GtkUIManager   *mgr,
+                                                         GtkActionGroup *actiongroup);
 
 void			rb_source_header_set_source	(RBSourceHeader *header,
 							 RBSource *player);
@@ -61,7 +62,7 @@ RBSource *		rb_source_header_get_source	(RBSourceHeader *shell_player);
 
 void			rb_source_header_clear_search	(RBSourceHeader *header);
 void			rb_source_header_sync_control_state (RBSourceHeader *header);
-
+void			rb_source_header_focus_search_box (RBSourceHeader *header);
 
 G_END_DECLS
 
