@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
- *  Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ *  Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,9 +55,24 @@
 extern char *mkdtemp (char *template);
 #endif
 
+/* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME was added in 2.12 */
+#ifndef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
+#define nautilus_burn_drive_eject _nautilus_burn_drive_eject
+#define nautilus_burn_drive_new_from_path _nautilus_burn_drive_new_from_path
+#define nautilus_burn_drive_media_type_get_string _nautilus_burn_drive_media_type_get_string
+#endif
+/* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME was added in 2.14 */
+#ifndef HAVE_BURN_DRIVE_UNREF
+#define nautilus_burn_drive_unref nautilus_burn_drive_free
+#define nautilus_burn_drive_ref nautilus_burn_drive_copy
+#endif
+
 #include "rb-recorder.h"
 
+#define CONF_STATE_BURN_SPEED CONF_PREFIX "/state/burn_speed"
+
 #define AUDIO_BYTERATE (2 * 44100 * 2)
+#define MAX_PLAYLIST_DURATION 6000
 
 static void rb_playlist_source_recorder_class_init (RBPlaylistSourceRecorderClass *klass);
 static void rb_playlist_source_recorder_init       (RBPlaylistSourceRecorder *source);
@@ -88,7 +103,6 @@ struct RBPlaylistSourceRecorderPrivate
         RBRecorder  *recorder;
         GSList      *songs;
         GSList      *current;
-        GConfClient *gconf_client;
         GTimer      *timer;
         guint64      start_pos;
 
@@ -113,8 +127,6 @@ struct RBPlaylistSourceRecorderPrivate
         char        *tmp_dir;
 };
 
-static GObjectClass *parent_class = NULL;
-
 typedef enum {
         NAME_CHANGED,
         FILE_ADDED,
@@ -124,6 +136,8 @@ typedef enum {
 
 static guint rb_playlist_source_recorder_signals [LAST_SIGNAL] = { 0 };
 
+#define RB_PLAYLIST_SOURCE_RECORDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PLAYLIST_SOURCE_RECORDER, RBPlaylistSourceRecorderPrivate))
+
 G_DEFINE_TYPE(RBPlaylistSourceRecorder, rb_playlist_source_recorder, GTK_TYPE_DIALOG)
 
 static void
@@ -132,8 +146,8 @@ rb_playlist_source_recorder_style_set (GtkWidget *widget,
 {
         GtkDialog *dialog;
 
-        if (GTK_WIDGET_CLASS (parent_class)->style_set)
-                GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+        if (GTK_WIDGET_CLASS (rb_playlist_source_recorder_parent_class)->style_set)
+                GTK_WIDGET_CLASS (rb_playlist_source_recorder_parent_class)->style_set (widget, previous_style);
 
         dialog = GTK_DIALOG (widget);
 
@@ -151,14 +165,10 @@ rb_playlist_source_recorder_class_init (RBPlaylistSourceRecorderClass *klass)
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
-        widget_class->style_set    = rb_playlist_source_recorder_style_set;
+        widget_class->style_set = rb_playlist_source_recorder_style_set;
 
         object_class->finalize = rb_playlist_source_recorder_finalize;
 
-        g_type_class_add_private (klass, sizeof (RBPlaylistSourceRecorderPrivate));
-
         rb_playlist_source_recorder_signals [NAME_CHANGED] =
                 g_signal_new ("name_changed",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -181,6 +191,7 @@ rb_playlist_source_recorder_class_init (RBPlaylistSourceRecorderClass *klass)
                               1,
                               G_TYPE_STRING);
 
+        g_type_class_add_private (klass, sizeof (RBPlaylistSourceRecorderPrivate));
 }
 
 GtkWidget *
@@ -234,13 +245,38 @@ get_speed_selection (GtkWidget *combobox)
         return speed;
 }
 
+#ifndef HAVE_BURN_DRIVE_GET_WRITE_SPEEDS
+
+#define nautilus_burn_drive_get_write_speeds get_write_speeds
+
+static const int *
+get_write_speeds (NautilusBurnDrive *drive)
+{
+	int  max_speed;
+	int  i;
+        static int *write_speeds = NULL;
+
+	if (write_speeds == NULL) {
+		max_speed = drive->max_speed_write;
+		write_speeds = g_new0 (int, max_speed + 1);
+
+		for (i = 0; i < max_speed; i++) {
+			write_speeds [i] = max_speed - i;
+		}
+	}
+
+        return (const int*)write_speeds;
+}
+#endif
+
 static void
 update_speed_combobox (RBPlaylistSourceRecorder *source)
 {
         GtkWidget               *combobox;
-        GConfClient             *gc;
         char                    *name;
-        int                      i, default_speed;
+        int                      i;
+        int                      default_speed;
+	int                      default_speed_index;
         const NautilusBurnDrive *drive;
         GtkTreeModel            *model;
         GtkTreeIter              iter;
@@ -253,34 +289,43 @@ update_speed_combobox (RBPlaylistSourceRecorder *source)
         model = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox));
         gtk_list_store_clear (GTK_LIST_STORE (model));
 
-        gc = gconf_client_get_default ();
-        default_speed = gconf_client_get_int (gc, "/apps/nautilus-cd-burner/default_speed", NULL);
-        g_object_unref (G_OBJECT (gc));
-
         gtk_list_store_append (GTK_LIST_STORE (model), &iter);
         gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                             0, _("Maximum possible"),
                             1, 0,
                             -1);
 
+
+        default_speed = eel_gconf_get_integer (CONF_STATE_BURN_SPEED);
+        default_speed_index = 0;
+
         if (drive) {
-                for (i = 1; i <= drive->max_speed_write; i++) {
-                        name = g_strdup_printf ("%d \303\227", i);
+                const int *write_speeds;
+
+                write_speeds = nautilus_burn_drive_get_write_speeds ((NautilusBurnDrive *)drive);
+
+                for (i = 0; write_speeds [i] > 0; i++) {
+
+                        name = g_strdup_printf ("%d \303\227", write_speeds [i]);
+
+                        if (write_speeds [i] == default_speed) {
+                                default_speed_index = i + 1;
+                        }
 
                         gtk_list_store_append (GTK_LIST_STORE (model), &iter);
                         gtk_list_store_set (GTK_LIST_STORE (model), &iter,
                                             0, name,
-                                            1, i,
+                                            1, write_speeds [i],
                                             -1);
                         g_free (name);
                 }
 
-                /* Disable speed if max speed < 1 */
-                gtk_widget_set_sensitive (combobox, drive->max_speed_write > 0);
+                /* Disable speed if no items in list */
+                gtk_widget_set_sensitive (combobox, i > 0);
         }
 
         /* for now assume equivalence between index in comboxbox and speed */
-        gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), default_speed);
+        gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), default_speed_index);
 }
 
 void
@@ -303,6 +348,7 @@ set_media_device (RBPlaylistSourceRecorder *source)
         GError     *error;
 
         device = nautilus_burn_drive_selection_get_device (NAUTILUS_BURN_DRIVE_SELECTION (source->priv->device_menu));
+
         if (device && strcmp (device, "")) {
                 rb_recorder_set_device (source->priv->recorder, device, &error);
                 if (error) {
@@ -461,6 +507,89 @@ progress_set_fraction (GtkWidget *progress,
         gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), fraction);
 }
 
+#ifndef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
+/* copied from nautilus-burn-drive 2.12 */
+static gboolean
+_nautilus_burn_drive_eject (NautilusBurnDrive *drive)
+{
+        char    *cmd;
+        gboolean res;
+
+        g_return_val_if_fail (drive != NULL, FALSE);
+
+        if (drive->device == NULL)
+                return FALSE;
+        
+        cmd = g_strdup_printf ("eject %s", drive->device);
+        res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
+        g_free (cmd);
+
+        /* delay a bit to make sure eject finishes */
+        sleep (2);
+
+        return res;
+}
+/* copied from nautilus-burn-drive 2.12 */
+static NautilusBurnDrive *
+_nautilus_burn_drive_new_from_path (const char *device)
+{
+        GList             *drives, *l;
+        NautilusBurnDrive *drive;
+
+        drives = nautilus_burn_drive_get_list (FALSE, FALSE);
+
+        drive = NULL;
+
+        for (l = drives; l != NULL; l = l->next) {
+                NautilusBurnDrive *d = l->data;
+                if (g_str_equal (device, d->device)) {
+                        drive = nautilus_burn_drive_ref (d);
+                }
+        }
+
+        g_list_foreach (drives, (GFunc)nautilus_burn_drive_unref, NULL);
+        g_list_free (drives);
+
+        return drive;
+}
+
+/* copied from nautilus-burn-drive 2.12 */
+static const char *
+_nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
+{
+        switch (type) {
+        case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
+                return _("Could not determine media type because CD drive is busy");
+        case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
+                return _("Couldn't open media");
+        case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
+                return _("Unknown Media");
+        case NAUTILUS_BURN_MEDIA_TYPE_CD:
+                return _("Commercial CD or Audio CD");
+        case NAUTILUS_BURN_MEDIA_TYPE_CDR:
+                return _("CD-R");
+        case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
+                return _("CD-RW");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVD:
+                return _("DVD");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
+                return _("DVD-R, or DVD-RAM");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
+                return _("DVD-RW");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
+                return _("DVD-RAM");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
+                return _("DVD+R");
+        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
+                return _("DVD+RW");
+        default:
+                break;
+        }
+
+        return _("Broken media type");
+}
+#endif /* NAUTILUS_BURN_DRIVE_SIZE_TO_TIME */
+
 static int
 burn_cd (RBPlaylistSourceRecorder *source,
          GError                  **error)
@@ -472,7 +601,7 @@ burn_cd (RBPlaylistSourceRecorder *source,
 
         set_media_device (source);
 
-        set_message_text (source, _("Burning audio to CD"));
+        set_message_text (source, _("Writing audio to CD"));
 
         speed = get_speed_selection (source->priv->speed_combobox);
 
@@ -487,13 +616,21 @@ burn_cd (RBPlaylistSourceRecorder *source,
                 return RB_RECORDER_RESULT_ERROR;
 
         if (res == RB_RECORDER_RESULT_FINISHED) {
-                gboolean do_another;
-                const char *finished_msg;
+                NautilusBurnDrive *drive;
+                gboolean           do_another;
+                const char        *finished_msg;
 
                 finished_msg = _("Finished creating audio CD.");
 
                 rb_shell_hidden_notify (source->priv->shell, 0, finished_msg, source->priv->cd_icon, "");
 
+                /* save the write speed that was used */
+                eel_gconf_set_integer (CONF_STATE_BURN_SPEED, speed);
+
+                /* Always eject the disk after writing.  Too many drives mess up otherwise */
+                drive = (NautilusBurnDrive *)lookup_current_recorder (source);
+                nautilus_burn_drive_eject (drive);
+
                 do_another = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (source->priv->multiple_copies_checkbutton));
                 if (!do_another) {
                         set_message_text (source, finished_msg);
@@ -886,88 +1023,6 @@ burn_action_changed_cb (RBRecorder        *recorder,
                 set_message_text (source, text);
 }
 
-/* copied from nautilus-burn-drive 2.12 */
-static NautilusBurnDrive *
-_nautilus_burn_drive_new_from_path (const char *device)
-{
-        GList             *drives, *l;
-        NautilusBurnDrive *drive;
-
-        drives = nautilus_burn_drive_get_list (FALSE, FALSE);
-
-        drive = NULL;
-
-        for (l = drives; l != NULL; l = l->next) {
-                NautilusBurnDrive *d = l->data;
-                if (g_str_equal (device, d->device)) {
-                        drive = nautilus_burn_drive_copy (d);
-                }
-        }
-
-        g_list_foreach (drives, (GFunc)nautilus_burn_drive_free, NULL);
-        g_list_free (drives);
-
-        return drive;
-}
-
-/* copied from nautilus-burn-drive 2.12 */
-static const char *
-_nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
-{
-        switch (type) {
-        case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
-                return _("Could not determine media type because CD drive is busy");
-        case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
-                return _("Couldn't open media");
-        case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
-                return _("Unknown Media");
-        case NAUTILUS_BURN_MEDIA_TYPE_CD:
-                return _("Commercial CD or Audio CD");
-        case NAUTILUS_BURN_MEDIA_TYPE_CDR:
-                return _("CD-R");
-        case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
-                return _("CD-RW");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD:
-                return _("DVD");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
-                return _("DVD-R, or DVD-RAM");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
-                return _("DVD-RW");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
-                return _("DVD-RAM");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
-                return _("DVD+R");
-        case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
-                return _("DVD+RW");
-        default:
-                break;
-        }
-
-        return _("Broken media type");
-}
-
-/* copied from nautilus-burn-drive 2.12 */
-static gboolean
-_nautilus_burn_drive_eject (NautilusBurnDrive *drive)
-{
-        char    *cmd;
-        gboolean res;
-
-        g_return_val_if_fail (drive != NULL, FALSE);
-
-        if (drive->device == NULL)
-                return FALSE;
-        
-        cmd = g_strdup_printf ("eject %s", drive->device);
-        res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
-        g_free (cmd);
-
-        /* delay a bit to make sure eject finishes */
-        sleep (2);
-
-        return res;
-}
-
 static int
 ask_rewrite_disc (RBPlaylistSourceRecorder *source,
                   const char               *device)
@@ -980,11 +1035,11 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         char                 *msg;
         NautilusBurnDrive    *drive;
 
-        drive = _nautilus_burn_drive_new_from_path (device);
+        drive = nautilus_burn_drive_new_from_path (device);
         type = nautilus_burn_drive_get_media_type (drive);
 
         msg = g_strdup_printf (_("This %s appears to have information already recorded on it."),
-                               _nautilus_burn_drive_media_type_get_string (type));
+                               nautilus_burn_drive_media_type_get_string (type));
 
 
         dialog = gtk_message_dialog_new (GTK_WINDOW (source),
@@ -1020,10 +1075,10 @@ ask_rewrite_disc (RBPlaylistSourceRecorder *source,
         gtk_widget_destroy (dialog);
 
         if (res == RB_RECORDER_RESPONSE_RETRY) {
-                _nautilus_burn_drive_eject (drive);
+                nautilus_burn_drive_eject (drive);
         }
 
-        nautilus_burn_drive_free (drive);
+        nautilus_burn_drive_unref (drive);
 
         return res;
 }
@@ -1090,9 +1145,9 @@ rb_playlist_source_recorder_init (RBPlaylistSourceRecorder *source)
                           G_CALLBACK (delete_event_handler),
                           NULL);
 
-        source->priv = g_new0 (RBPlaylistSourceRecorderPrivate, 1);
-        source->priv->gconf_client = gconf_client_get_default ();
+        source->priv = RB_PLAYLIST_SOURCE_RECORDER_GET_PRIVATE (source);
 
+        gtk_window_set_resizable (GTK_WINDOW (source), FALSE);
         gtk_dialog_set_has_separator (GTK_DIALOG (source), FALSE);
         source->priv->cancel_button =  gtk_dialog_add_button (GTK_DIALOG (source),
                                                               GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
@@ -1134,6 +1189,7 @@ rb_playlist_source_recorder_init (RBPlaylistSourceRecorder *source)
         source->priv->progress = glade_xml_get_widget (xml, "progress");
         gtk_progress_bar_set_ellipsize (GTK_PROGRESS_BAR (source->priv->progress), PANGO_ELLIPSIZE_END);
         gtk_progress_bar_set_text (GTK_PROGRESS_BAR (source->priv->progress), " ");
+        gtk_widget_set_size_request (source->priv->progress, 400, -1);
 
         source->priv->progress_frame = glade_xml_get_widget (xml, "progress_frame");
 
@@ -1147,19 +1203,8 @@ rb_playlist_source_recorder_init (RBPlaylistSourceRecorder *source)
         widget = glade_xml_get_widget (xml, "device_label");
         gtk_label_set_mnemonic_widget (GTK_LABEL (widget), source->priv->device_menu);
 
-        pattrlist = pango_attr_list_new ();
-        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        widget = glade_xml_get_widget (xml, "progress_frame_label");
-        gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);
-
-        widget = glade_xml_get_widget (xml, "options_expander_label");
-        gtk_label_set_attributes (GTK_LABEL (widget), pattrlist);
-
-        pango_attr_list_unref (pattrlist);
+	rb_glade_boldify_label (xml, "progress_frame_label");
+	rb_glade_boldify_label (xml, "options_expander_label");
 
         pattrlist = pango_attr_list_new ();
         attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
@@ -1244,11 +1289,23 @@ recorder_song_free (RBRecorderSong *song)
         g_free (song);
 }
 
+static void
+free_song_list (GSList *songs)
+{
+        GSList *l;
+
+        for (l = songs; l; l = l->next) {
+                recorder_song_free ((RBRecorderSong *)l->data);
+        }
+
+        g_slist_free (songs);
+        songs = NULL;
+}
+
 static void
 rb_playlist_source_recorder_finalize (GObject *object)
 {
         RBPlaylistSourceRecorder *source;
-        GSList *l;
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (RB_IS_PLAYLIST_SOURCE_RECORDER (object));
@@ -1263,18 +1320,10 @@ rb_playlist_source_recorder_finalize (GObject *object)
 
         g_object_unref (source->priv->cd_icon);
 
-        if (source->priv->gconf_client)
-                g_object_unref (source->priv->gconf_client);
-        source->priv->gconf_client = NULL;
-
-        for (l = source->priv->songs; l; l = l->next)
-                recorder_song_free ((RBRecorderSong *)l->data);
-
         g_free (source->priv->name);
         source->priv->name = NULL;
 
-        g_slist_free (source->priv->songs);
-        source->priv->songs = NULL;
+        free_song_list (source->priv->songs);
 
         g_object_unref (source->priv->recorder);
         source->priv->recorder = NULL;
@@ -1288,10 +1337,7 @@ rb_playlist_source_recorder_finalize (GObject *object)
                 source->priv->tmp_dir = NULL;
         }
 
-        g_free (source->priv);
-        source->priv = NULL;
-
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (rb_playlist_source_recorder_parent_class)->finalize (object);
 }
 
 GtkWidget *
@@ -1345,35 +1391,81 @@ rb_playlist_source_recorder_set_name (RBPlaylistSourceRecorder *source,
                        name);
 }
 
-void
+gboolean
 rb_playlist_source_recorder_add_from_model (RBPlaylistSourceRecorder *source,
                                             GtkTreeModel             *model,
                                             RBPlaylistSourceIterFunc  func,
                                             GError                  **error)
 {
         GtkTreeIter iter;
+        gboolean    failed;
+        GSList     *songs  = NULL;
+        GSList     *l;
+        guint64     length = 0;
 
-        g_return_if_fail (source != NULL);
-        g_return_if_fail (RB_IS_PLAYLIST_SOURCE_RECORDER (source));
+        g_return_val_if_fail (source != NULL, FALSE);
+        g_return_val_if_fail (RB_IS_PLAYLIST_SOURCE_RECORDER (source), FALSE);
 
-        g_return_if_fail (model != NULL);
+        g_return_val_if_fail (model != NULL, FALSE);
 
-        if (!gtk_tree_model_get_iter_first (model, &iter))
-                return;
+        if (! gtk_tree_model_get_iter_first (model, &iter)) {
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
+                             RB_RECORDER_ERROR_GENERAL,
+                             _("Unable to build an audio track list."));
 
+                return FALSE;
+        }
+
+        /* Make sure we can use all of the songs before we
+           modify the song list */
+        failed = FALSE;
         do {
                 RBRecorderSong *song = recorder_song_new ();
-                
-                func (model, &iter, &song->uri, &song->artist, &song->title, &song->duration);
+                gboolean        res;
+
+                res = func (model, &iter, &song->uri, &song->artist, &song->title, &song->duration);
+                if (! res) {
+                        failed = TRUE;
+                        g_set_error (error,
+                                     RB_RECORDER_ERROR,
+                                     RB_RECORDER_ERROR_GENERAL,
+                                     _("Unable to build an audio track list."));
+                        break;
+                }
+
+                length += song->duration;
+                if (length > MAX_PLAYLIST_DURATION) {
+                        failed = TRUE;
+                        g_set_error (error,
+                                     RB_RECORDER_ERROR,
+                                     RB_RECORDER_ERROR_GENERAL,
+                                     _("This playlist is too long to write to an audio CD."));
+                        break;
+                }
+
+                songs = g_slist_append (songs, song);
+        } while (gtk_tree_model_iter_next (model, &iter));
+
+        if (failed) {
+                free_song_list (songs);
+
+                return FALSE;
+        }
+
+        /* now that we've checked all the songs, add them to the song list */
+        for (l = songs; l; l = l->next) {
+                RBRecorderSong *song = l->data;
 
                 source->priv->songs = g_slist_append (source->priv->songs, song);
+
                 g_signal_emit (G_OBJECT (source),
                                rb_playlist_source_recorder_signals [FILE_ADDED],
                                0,
                                song->uri);
-        } while (gtk_tree_model_iter_next (model, &iter));
+        }
 
-        return;
+        return TRUE;
 }
 
 static guint64
@@ -1489,6 +1581,41 @@ check_tmp_dir (RBPlaylistSourceRecorder *source,
         return TRUE;
 }
 
+static gboolean
+check_media_length (RBPlaylistSourceRecorder *source,
+                    GError                  **error)
+{
+        gint64  duration = rb_playlist_source_recorder_get_total_duration (source);
+        char   *message  = NULL;
+        gint64  media_duration;
+        char   *duration_string;
+
+        media_duration = rb_recorder_get_media_length (source->priv->recorder, NULL);
+        duration_string = g_strdup_printf ("%" G_GINT64_FORMAT, duration / 60);
+
+        /* Only check if the playlist is greater than 74 minutes */
+        if ((media_duration < 0) && (duration > 4440)) {
+                message = g_strdup_printf (_("This playlist is %s minutes long.  "
+                                             "This exceeds the length of a standard audio CD.  "
+                                             "If the destination media is larger than a standard audio CD "
+                                             "please insert it in the drive and try again."),
+                                           duration_string);
+        }
+
+        g_free (duration_string);
+
+        if (message) {
+                error_dialog (source,
+                              _("Playlist too long"),
+                              message);
+                g_free (message);
+
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
 void
 rb_playlist_source_recorder_start (RBPlaylistSourceRecorder *source,
                                    GError                  **error)
@@ -1504,41 +1631,17 @@ rb_playlist_source_recorder_start (RBPlaylistSourceRecorder *source,
         if (source->priv->already_converted) {
                 g_idle_add ((GSourceFunc)burn_cd_idle, source);
         } else {
-                gint64  duration = rb_playlist_source_recorder_get_total_duration (source);
-                char   *message  = NULL;
-                gint64  media_duration;
-                char   *duration_string;
+                gboolean is_ok;
 
                 set_media_device (source);
 
-                media_duration = rb_recorder_get_media_length (source->priv->recorder, NULL);
-                duration_string = g_strdup_printf ("%" G_GINT64_FORMAT, duration / 60);
-
-                if ((media_duration < 0) && (duration > 4440)) {
-                        message = g_strdup_printf (_("This playlist is %s minutes long.  "
-                                                     "This exceeds the length of a standard audio CD.  "
-                                                     "If the destination media is larger than a standard audio CD "
-                                                     "please insert it in the drive and try again."),
-                                                   duration_string);
-                } else if ((media_duration > 0) && (media_duration <= duration)) {
-                        char *media_duration_string = g_strdup_printf ("%" G_GINT64_FORMAT, media_duration / 60);
-                        message = g_strdup_printf (_("This playlist is %s minutes long.  "
-                                                     "This exceeds the %s minute length of the media in the drive."),
-                                                   duration_string, media_duration_string);
-                        g_free (media_duration_string);
-                }
-                g_free (duration_string);
-
-                if (message) {
-                        error_dialog (source,
-                                      _("Playlist too long"),
-                                      message);
-                        g_free (message);
-
+                is_ok = check_media_length (source, error);
+                if (! is_ok) {
                         return;
                 }
 
-                if (!check_tmp_dir (source, error)) {
+                is_ok = check_tmp_dir (source, error);
+                if (! is_ok) {
                         guint64 mib_needed = rb_playlist_source_recorder_estimate_total_size (source) / 1048576;
                         char   *mib_needed_string = g_strdup_printf ("%" G_GUINT64_FORMAT, mib_needed);
 
