@@ -23,21 +23,26 @@
 
 #include "config.h"
 
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <libgnome/gnome-i18n.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+
 #include "rb-sourcelist-model.h"
 #include "rb-tree-dnd.h"
 #include "rb-debug.h"
 #include "rb-marshal.h"
 #include "rb-playlist-source.h"
 
-struct RBSourceListModelPriv
+/* Some compilers do not like empty structures, noop is not used */
+struct RBSourceListModelPrivate
 {
+   int noop;
 };
 
+#define RB_SOURCELIST_MODEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCELIST_MODEL, RBSourceListModelPrivate))
+
 enum
 {
 	DROP_RECEIVED,
@@ -129,6 +134,8 @@ rb_sourcelist_model_class_init (RBSourceListModelClass *class)
 		sourcelist_drag_target_list = 
 			gtk_target_list_new (sourcelist_targets,
 					     G_N_ELEMENTS (sourcelist_targets));
+
+	g_type_class_add_private (class, sizeof (RBSourceListModelPrivate));
 }
 
 static void
@@ -156,29 +163,25 @@ rb_sourcelist_model_set_dnd_targets (RBSourceListModel *sourcelist,
 	g_return_if_fail (RB_IS_SOURCELIST_MODEL (sourcelist));
 
 	rb_tree_dnd_add_drag_dest_support (treeview,
-					   RB_TREE_DEST_EMPTY_VIEW_DROP,
+					   (RB_TREE_DEST_EMPTY_VIEW_DROP | RB_TREE_DEST_SELECT_ON_DRAG_TIMEOUT),
 					   sourcelist_targets, n_targets,
 					   GDK_ACTION_LINK);
 
 	rb_tree_dnd_add_drag_source_support (treeview,
 					     GDK_BUTTON1_MASK,
 					     sourcelist_targets, n_targets,
-					     GDK_ACTION_MOVE | GDK_ACTION_COPY);
+					     GDK_ACTION_COPY);
 }
 		
 static void
 rb_sourcelist_model_init (RBSourceListModel *model)
 {
-	model->priv = g_new0 (RBSourceListModelPriv, 1);
+	model->priv = RB_SOURCELIST_MODEL_GET_PRIVATE (model);
 }
 
 static void
 rb_sourcelist_model_finalize (GObject *object)
 {
-	RBSourceListModel *model = RB_SOURCELIST_MODEL (object);
-
-	g_free (model->priv);
-
 	G_OBJECT_CLASS (rb_sourcelist_model_parent_class)->finalize (object);
 }
 
@@ -215,6 +218,7 @@ rb_sourcelist_model_new (void)
 	column_types[RB_SOURCELIST_MODEL_COLUMN_NAME] = G_TYPE_STRING;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_SOURCE] = G_TYPE_POINTER;
 	column_types[RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES] = PANGO_TYPE_ATTR_LIST;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_VISIBILITY] = G_TYPE_BOOLEAN;
 	store = gtk_tree_store_newv (RB_SOURCELIST_MODEL_N_COLUMNS, 
 				     column_types);
 
@@ -343,7 +347,7 @@ path_is_droppable (RBSourceListModel *model,
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
 		
-		return rb_source_can_cut (source);
+		return rb_source_can_paste (source);
 
 	}
 	return FALSE;
@@ -496,10 +500,10 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		return TRUE;
 	} else if (target == TARGET_URIS) {
 		RBSource *source;
-		RBEntryView *view;
-		RhythmDBEntry *entry, *first_entry;
+		RhythmDBQueryModel *query_model;
 		GtkTreeIter iter;
 		GString *data;
+		gboolean first = TRUE;
 
 		rb_debug ("getting drag data as uri list");
 		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source), &iter, path))
@@ -508,20 +512,26 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 		data = g_string_new ("");
 		gtk_tree_model_get (GTK_TREE_MODEL (drag_source), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
-		view = rb_source_get_entry_view (source);
-		first_entry = rb_entry_view_get_first_entry (view);
-		if (!first_entry)
+		g_object_get (G_OBJECT (source), "query-model", &query_model, NULL);
+
+		if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (query_model), &iter)) {
+			g_object_unref (G_OBJECT (query_model));
 			return FALSE;
+		}
 
-		entry = first_entry;
 		do {
-			if (entry != first_entry)
-				g_string_append (data, "\r\n");
-			g_string_append (data, entry->location);
+			RhythmDBEntry *entry;
+			if (first) {
+				g_string_append(data, "\r\n");
+				first = FALSE;
+			}
 
-			entry = rb_entry_view_get_next_from_entry (view, entry);
-		} while (entry && entry != first_entry);
+			entry = rhythmdb_query_model_iter_to_entry (query_model, &iter);
+			g_string_append (data, rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
 
+		} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (query_model), &iter));
+		g_object_unref (G_OBJECT (query_model));
+		
 		gtk_selection_data_set (selection_data,
 					selection_data->target,
 					8, (guchar *) data->str,
