@@ -37,6 +37,7 @@
 #include "rhythmdb-property-model.h"
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
+#include "rb-util.h"
 
 static void rb_property_view_class_init (RBPropertyViewClass *klass);
 static void rb_property_view_init (RBPropertyView *view);
@@ -62,6 +63,11 @@ static void rb_property_view_pre_row_deleted_cb (RhythmDBPropertyModel *model,
 static void rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 						  GtkTreePath *path,
 						  RBPropertyView *view);
+static gboolean rb_property_view_popup_menu_cb (GtkTreeView *treeview,
+						RBPropertyView *view);
+static gboolean rb_property_view_button_press_cb (GtkTreeView *tree,
+						  GdkEventButton *event,
+						  RBPropertyView *view);
 
 struct RBPropertyViewPrivate
 {
@@ -88,6 +94,7 @@ enum
 	PROPERTIES_SELECTED,
 	PROPERTY_ACTIVATED,
 	SELECTION_RESET,
+	SHOW_POPUP,
 	LAST_SIGNAL
 };
 
@@ -150,7 +157,7 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 					 g_param_spec_enum ("prop",
 							    "PropertyId",
 							    "RhythmDBPropType",
-							    RHYTHMDB_TYPE_PROP,
+							    RHYTHMDB_TYPE_PROP_TYPE,
 							    RHYTHMDB_PROP_TYPE,
 							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
@@ -217,7 +224,17 @@ rb_property_view_class_init (RBPropertyViewClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
-			      0); 
+			      0);
+	rb_property_view_signals[SHOW_POPUP] =
+		g_signal_new ("show_popup",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPropertyViewClass, show_popup),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 
 	g_type_class_add_private (klass, sizeof (RBPropertyViewPrivate));
 }
@@ -303,9 +320,9 @@ rb_property_view_set_property (GObject *object,
 						 G_CALLBACK (rb_property_view_selection_changed_cb),
 						 view);
 		
-		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter);
 		gtk_tree_selection_unselect_all (view->priv->selection);
-		gtk_tree_selection_select_iter (view->priv->selection, &iter);
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &iter))
+			gtk_tree_selection_select_iter (view->priv->selection, &iter);
 		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
 						   G_CALLBACK (rb_property_view_selection_changed_cb),
 						   view);
@@ -429,16 +446,16 @@ rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 	if (gtk_tree_selection_count_selected_rows (view->priv->selection) == 0) {
 		GtkTreeIter first_iter;
 		rb_debug ("no rows selected, signalling reset");
-		gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model),
-					       &first_iter);
-		g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
-						 G_CALLBACK (rb_property_view_selection_changed_cb),
-						 view);
-		gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
-		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
-		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
-						   G_CALLBACK (rb_property_view_selection_changed_cb),
-						   view);
+		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &first_iter)) {
+			g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
+							 G_CALLBACK (rb_property_view_selection_changed_cb),
+							 view);
+			gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
+			g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
+			g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
+							   G_CALLBACK (rb_property_view_selection_changed_cb),
+							   view);
+		}
 	}
 }
 
@@ -536,6 +553,18 @@ rb_property_view_constructor (GType type,
 			         G_CALLBACK (rb_property_view_selection_changed_cb),
 			         view,
 				 0);
+	g_signal_connect_object (G_OBJECT (view->priv->treeview),
+				 "popup_menu",
+				 G_CALLBACK (rb_property_view_popup_menu_cb),
+				 view,
+				 0);
+	
+	g_signal_connect_object (G_OBJECT (view->priv->treeview),
+			         "button_press_event",
+			         G_CALLBACK (rb_property_view_button_press_cb),
+			         view,
+				 0);
+
 
 	gtk_container_add (GTK_CONTAINER (view), view->priv->treeview);
 
@@ -567,7 +596,8 @@ rb_property_view_row_activated_cb (GtkTreeView *treeview,
 	gboolean is_all;
 
 	rb_debug ("row activated");
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->prop_model), &iter, path);
+	g_return_if_fail (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->prop_model),
+			  &iter, path));
 
 	gtk_tree_model_get (GTK_TREE_MODEL (view->priv->prop_model), &iter,
 			    RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE, &val,
@@ -647,14 +677,15 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 							 G_CALLBACK (rb_property_view_selection_changed_cb),
 							 view);
 			gtk_tree_selection_unselect_all (selection);
-			gtk_tree_model_get_iter_first (model, &iter);
-			gtk_tree_selection_select_iter (selection, &iter);
+			if (gtk_tree_model_get_iter_first (model, &iter))
+				gtk_tree_selection_select_iter (selection, &iter);
 			g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
 							   G_CALLBACK (rb_property_view_selection_changed_cb),
 							   view);
 		}
 		g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTIES_SELECTED], 0,
 			       selected_properties);
+		rb_list_deep_free (selected_properties);
 	} else {
 		if (gtk_tree_selection_get_selected (view->priv->selection, &model, &iter)) {
 			gtk_tree_model_get (model, &iter,
@@ -666,9 +697,65 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 	}
 }
 
-GtkWidget*
-rb_property_view_get_treeview	(RBPropertyView *view)
+static gboolean 
+rb_property_view_popup_menu_cb (GtkTreeView *treeview,
+				RBPropertyView *view)
 {
-	return view->priv->treeview;
+	g_signal_emit (G_OBJECT (view), rb_property_view_signals[SHOW_POPUP], 0);
+	return TRUE;
 }
 
+void		
+rb_property_view_append_column_custom (RBPropertyView *view,
+				       GtkTreeViewColumn *column)
+{
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
+}
+
+static gboolean
+rb_property_view_button_press_cb (GtkTreeView *tree,
+				  GdkEventButton *event,
+				  RBPropertyView *view)
+{
+
+	if (event->button == 3) {
+		GtkTreeSelection *selection;
+		GtkTreePath *path;
+
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
+
+		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (view->priv->treeview), event->x, event->y, &path, NULL, NULL, NULL);
+		if (path == NULL) {
+			gtk_tree_selection_unselect_all (selection);
+		} else {
+			GtkTreeModel *model;
+			GtkTreeIter iter;
+			const char *val;
+			GList *lst = NULL;
+			
+			model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->priv->treeview));
+			if (gtk_tree_model_get_iter (model, &iter, path)) {
+				gtk_tree_model_get (model, &iter, 0, &val, -1);
+				lst = g_list_prepend (lst, (gpointer) val);
+				rb_property_view_set_selection (view, lst);
+			}
+		}
+		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SHOW_POPUP], 0);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+void
+rb_property_view_set_search_func (RBPropertyView *view,
+				  GtkTreeViewSearchEqualFunc func,
+				  gpointer func_data,
+				  GtkDestroyNotify notify)
+{
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (view->priv->treeview),
+					     func, func_data,
+					     notify);
+}
+
+
