@@ -37,8 +37,8 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
+#include "rhythmdb-property-model.h"
 #include "rb-song-info.h"
-#include "rb-enums.h"
 #include "rb-glade-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
@@ -123,6 +123,10 @@ struct RBSongInfoPrivate
 	GtkWidget   *play_count;
 	GtkWidget   *last_played;
 	GtkWidget   *rating;
+
+	RhythmDBPropertyModel* albums;
+	RhythmDBPropertyModel* artists;
+	RhythmDBPropertyModel* genres;
 };
 
 #define RB_SONG_INFO_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SONG_INFO, RBSongInfoPrivate))
@@ -295,6 +299,18 @@ rb_song_info_construct_multiple (RBSongInfo *song_info, GladeXML *xml,
 	gtk_widget_grab_focus (song_info->priv->artist);
 }
 
+static void
+rb_song_info_add_completion (GtkEntry *entry, RhythmDBPropertyModel *propmodel)
+{
+	GtkEntryCompletion* completion;
+
+	completion = gtk_entry_completion_new();
+	gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (propmodel));
+	gtk_entry_completion_set_text_column (completion, RHYTHMDB_PROPERTY_MODEL_COLUMN_TITLE);
+	gtk_entry_set_completion (entry, completion);
+	g_object_unref (completion);
+}
+
 static GObject *
 rb_song_info_constructor (GType type, guint n_construct_properties,
 			  GObjectConstructParam *construct_properties)
@@ -357,6 +373,10 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	song_info->priv->playback_error_label = glade_xml_get_widget (xml, "song_info_error_label");
 	song_info->priv->disc_cur = glade_xml_get_widget (xml, "song_info_disc_cur");
 
+	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->genre), song_info->priv->genres);
+	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->artist), song_info->priv->artists);
+	rb_song_info_add_completion (GTK_ENTRY (song_info->priv->album), song_info->priv->albums);
+	
 	rb_glade_boldify_label (xml, "album_label");
 	rb_glade_boldify_label (xml, "artist_label");
 	rb_glade_boldify_label (xml, "genre_label");
@@ -435,6 +455,10 @@ rb_song_info_finalize (GObject *object)
 					      G_CALLBACK (rb_song_info_query_model_changed_cb),
 					      song_info);
 
+	g_object_unref (song_info->priv->albums);
+	g_object_unref (song_info->priv->artists);
+	g_object_unref (song_info->priv->genres);
+
 	g_object_unref (G_OBJECT (song_info->priv->db));
 	g_object_unref (G_OBJECT (song_info->priv->source));
 	g_object_unref (G_OBJECT (song_info->priv->query_model));
@@ -454,17 +478,54 @@ rb_song_info_set_property (GObject *object,
 	{
 	case PROP_SOURCE:
 	{
+		RhythmDB* old_db = song_info->priv->db;
+
 		song_info->priv->source = g_value_get_object (value);
 		g_object_ref (G_OBJECT (song_info->priv->source));
-		g_object_get (G_OBJECT (song_info->priv->source), 
+		g_object_get (G_OBJECT (song_info->priv->source),
 			      "query-model", &song_info->priv->query_model, NULL);
 		g_signal_connect_object (G_OBJECT (song_info->priv->source),
 					 "notify::query-model",
 					 G_CALLBACK (rb_song_info_query_model_changed_cb),
 					 song_info, 0);
 
-		g_object_get (G_OBJECT (song_info->priv->query_model), "db", 
+		g_object_get (G_OBJECT (song_info->priv->query_model), "db",
 			      &song_info->priv->db, NULL);
+
+		if(old_db != song_info->priv->db) {
+			if(song_info->priv->albums) {
+				g_object_unref(song_info->priv->albums);
+			}
+			if(song_info->priv->artists) {
+				g_object_unref(song_info->priv->artists);
+			}
+			if(song_info->priv->genres) {
+				g_object_unref(song_info->priv->genres);
+			}
+
+			song_info->priv->albums  = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_ALBUM);
+			song_info->priv->artists = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_ARTIST);
+			song_info->priv->genres  = rhythmdb_property_model_new(song_info->priv->db, RHYTHMDB_PROP_GENRE);
+
+			g_object_set(song_info->priv->albums,  "query-model", song_info->priv->query_model, NULL);
+			g_object_set(song_info->priv->artists, "query-model", song_info->priv->query_model, NULL);
+			g_object_set(song_info->priv->genres,  "query-model", song_info->priv->query_model, NULL);
+
+			if(song_info->priv->album) {
+				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->album));
+				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->albums));
+			}
+
+			if(song_info->priv->artist) {
+				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->artist));
+				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->artist));
+			}
+
+			if(song_info->priv->genre) {
+				GtkEntryCompletion* comp = gtk_entry_get_completion (GTK_ENTRY (song_info->priv->genre));
+				gtk_entry_completion_set_model (comp, GTK_TREE_MODEL (song_info->priv->genre));
+			}
+		}
 	}
 	break;
 	case PROP_ENTRY_VIEW:
@@ -544,8 +605,7 @@ rb_song_info_response_cb (GtkDialog *dialog,
 			  RBSongInfo *song_info)
 {
 	if (response_id == GTK_RESPONSE_CLOSE) {
-		if (song_info->priv->editable)
-			rb_song_info_sync_entries (RB_SONG_INFO (dialog));
+		rb_song_info_sync_entries (RB_SONG_INFO (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	}
 }
@@ -640,7 +700,7 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 		entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
 		entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
 		entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
-		entry_year = (entry->date) ? g_date_get_year (entry->date) : 0;
+		entry_year = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
 		entry_disc_number = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
 
 		/* grab first valid values */
@@ -688,32 +748,33 @@ rb_song_info_populate_dialog_multiple (RBSongInfo *song_info)
 static void 
 rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
-	const char *text = NULL;
+	const char *text;
 	char *tmp;
+	ulong num;
 	
 	g_assert (song_info->priv->current_entry);
 
 	/* update the buttons sensitivity */
 	rb_song_info_update_buttons (song_info);
 
-	text = rb_refstring_get (song_info->priv->current_entry->title);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_TITLE);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
 	
 	tmp = g_strdup_printf (_("%s Properties"), text);
 	gtk_window_set_title (GTK_WINDOW (song_info), tmp);
 	g_free (tmp);
 
-	text = rb_refstring_get (song_info->priv->current_entry->artist);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ARTIST);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), text);
-	text = rb_refstring_get (song_info->priv->current_entry->album);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_ALBUM);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), text);
-	text = rb_refstring_get (song_info->priv->current_entry->genre);
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_GENRE);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), text);
 
-	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->track_cur),
-					 song_info->priv->current_entry->tracknum);
-	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur),
-					 song_info->priv->current_entry->discnum);
+	num = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_TRACK_NUMBER);
+	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->track_cur), num);
+	num = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_DISC_NUMBER);
+	rb_song_info_populate_num_field (GTK_ENTRY (song_info->priv->disc_cur), num);
 
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
@@ -733,8 +794,7 @@ rb_song_info_update_playback_error (RBSongInfo *song_info)
 	if (!song_info->priv->current_entry)
 		return;
 
-	if (song_info->priv->current_entry->playback_error)
-		message = g_strdup (song_info->priv->current_entry->playback_error);
+	message = rhythmdb_entry_dup_string (song_info->priv->current_entry, RHYTHMDB_PROP_PLAYBACK_ERROR);
 
 	if (message) {
 		gtk_label_set_text (GTK_LABEL (song_info->priv->playback_error_label),
@@ -754,7 +814,7 @@ rb_song_info_update_bitrate (RBSongInfo *song_info)
 {
 	char *tmp = NULL;
 	gulong bitrate = 0;
-	bitrate = song_info->priv->current_entry->bitrate;
+	bitrate = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_BITRATE);
 
 	if (bitrate > 0)
 		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
@@ -771,7 +831,7 @@ rb_song_info_update_duration (RBSongInfo *song_info)
 	char *text = NULL;
 	long duration = 0;
 	int minutes, seconds;
-	duration = song_info->priv->current_entry->duration;
+	duration = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_DURATION);
 	minutes = duration / 60;
 	seconds = duration % 60;
 	text = g_strdup_printf ("%d:%02d", minutes, seconds);
@@ -786,7 +846,7 @@ rb_song_info_update_location (RBSongInfo *song_info)
 
 	g_return_if_fail (song_info != NULL);
 
-	text = song_info->priv->current_entry->location;
+	text = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_LOCATION);
 
 	if (text != NULL) {
 		char *tmp, *tmp_utf8;
@@ -840,15 +900,17 @@ static void
 rb_song_info_backward_clicked_cb (GtkWidget *button,
 				  RBSongInfo *song_info)
 {
+	RhythmDBEntry *new_entry;
+
 	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
-	song_info->priv->current_entry =
+	new_entry = 
 		rhythmdb_query_model_get_previous_from_entry (song_info->priv->query_model,
 							      song_info->priv->current_entry);
+	g_return_if_fail (new_entry != NULL);
 
-	rb_entry_view_select_entry (song_info->priv->entry_view,
-				    song_info->priv->current_entry);
-	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
-				       song_info->priv->current_entry);
+	song_info->priv->current_entry = new_entry;
+	rb_entry_view_select_entry (song_info->priv->entry_view, new_entry);
+	rb_entry_view_scroll_to_entry (song_info->priv->entry_view, new_entry);
 
 	rb_song_info_populate_dialog (song_info);
 }
@@ -857,15 +919,17 @@ static void
 rb_song_info_forward_clicked_cb (GtkWidget *button,
 				 RBSongInfo *song_info)
 {
+	RhythmDBEntry *new_entry;
+
 	rb_song_info_sync_entries (RB_SONG_INFO (song_info));
-	song_info->priv->current_entry =
+	new_entry = 
 		rhythmdb_query_model_get_next_from_entry (song_info->priv->query_model,
 							  song_info->priv->current_entry);
+	g_return_if_fail (new_entry != NULL);
 
-	rb_entry_view_select_entry (song_info->priv->entry_view,
-				    song_info->priv->current_entry);
-	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
-				       song_info->priv->current_entry);
+	song_info->priv->current_entry = new_entry;
+	rb_entry_view_select_entry (song_info->priv->entry_view, new_entry);
+	rb_entry_view_scroll_to_entry (song_info->priv->entry_view, new_entry);
 
 	rb_song_info_populate_dialog (song_info);
 }
@@ -896,15 +960,67 @@ rb_song_info_update_buttons (RBSongInfo *song_info)
 	gtk_widget_set_sensitive (song_info->priv->forward, entry != NULL);
 }
 
+static void
+rb_song_info_query_model_inserted_cb (RhythmDBQueryModel *model,
+				      GtkTreePath *path,
+				      GtkTreeIter *iter,
+				      RBSongInfo *song_info)
+{
+	rb_song_info_update_buttons (song_info);
+}
+
+static void
+rb_song_info_query_model_deleted_cb (RhythmDBQueryModel *model,
+				     RhythmDBEntry*entry,
+				     RBSongInfo *song_info)
+{
+	rb_song_info_update_buttons (song_info);
+}
+
+static void
+rb_song_info_query_model_reordered_cb (RhythmDBQueryModel *model,
+				       GtkTreePath *path,
+				       GtkTreeIter *iter,
+				       gpointer *map,
+				       RBSongInfo *song_info)
+{
+	rb_song_info_update_buttons (song_info);
+}
+
 static void
 rb_song_info_query_model_changed_cb (GObject *source,
 				     GParamSpec *whatever,
 				     RBSongInfo *song_info)
 {
-	if (song_info->priv->query_model)
+	if (song_info->priv->query_model) {
+		g_signal_handlers_disconnect_by_func (G_OBJECT (song_info->priv->query_model),
+						      G_CALLBACK (rb_song_info_query_model_inserted_cb),
+						      song_info);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (song_info->priv->query_model),
+						      G_CALLBACK (rb_song_info_query_model_deleted_cb),
+						      song_info);
+		g_signal_handlers_disconnect_by_func (G_OBJECT (song_info->priv->query_model),
+						      G_CALLBACK (rb_song_info_query_model_reordered_cb),
+						      song_info);
+
 		g_object_unref (G_OBJECT (song_info->priv->query_model));
+	}
 
 	g_object_get (source, "query-model", &song_info->priv->query_model, NULL);
+
+	g_signal_connect_object (G_OBJECT (song_info->priv->query_model),
+				 "row-inserted", G_CALLBACK (rb_song_info_query_model_inserted_cb),
+				 song_info, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->query_model),
+				 "row-changed", G_CALLBACK (rb_song_info_query_model_inserted_cb),
+				 song_info, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->query_model),
+				 "entry-deleted", G_CALLBACK (rb_song_info_query_model_deleted_cb),
+				 song_info, 0);
+	g_signal_connect_object (G_OBJECT (song_info->priv->query_model),
+				 "rows-reordered", G_CALLBACK (rb_song_info_query_model_reordered_cb),
+				 song_info, 0);
+	
 	/* update next button sensitivity */
 	rb_song_info_update_buttons (song_info);
 }
@@ -912,8 +1028,11 @@ rb_song_info_query_model_changed_cb (GObject *source,
 static void
 rb_song_info_update_play_count (RBSongInfo *song_info)
 {
+	gulong num;
 	char *text;
-	text = g_strdup_printf ("%ld", song_info->priv->current_entry->play_count);
+
+	num = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_PLAY_COUNT);
+	text = g_strdup_printf ("%ld", num);
 	gtk_label_set_text (GTK_LABEL (song_info->priv->play_count), text);
 	g_free (text);
 }
@@ -922,7 +1041,7 @@ static void
 rb_song_info_update_last_played (RBSongInfo *song_info)
 {
 	const char *str;
-	str = rb_refstring_get (song_info->priv->current_entry->last_played_str);
+	str = rhythmdb_entry_get_string (song_info->priv->current_entry, RHYTHMDB_PROP_LAST_PLAYED_STR);
 	if (!strcmp ("", str))
 		str = _("Never");
 	gtk_label_set_text (GTK_LABEL (song_info->priv->last_played), str);
@@ -931,21 +1050,24 @@ rb_song_info_update_last_played (RBSongInfo *song_info)
 static void
 rb_song_info_update_rating (RBSongInfo *song_info)
 {
+	double rating;
+
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
+	rating = rhythmdb_entry_get_double (song_info->priv->current_entry, RHYTHMDB_PROP_RATING);
 	g_object_set (G_OBJECT (song_info->priv->rating),
-		      "rating", song_info->priv->current_entry->rating,
+		      "rating", rating,
 		      NULL);
-
 }
 
 static void
 rb_song_info_update_year (RBSongInfo *song_info)
 {
+	ulong year;
 	char *text;
 
-	if (song_info->priv->current_entry->date) {
-		gulong year = g_date_get_year (song_info->priv->current_entry->date);
+	year = rhythmdb_entry_get_ulong (song_info->priv->current_entry, RHYTHMDB_PROP_YEAR);
+	if (year > 0) {
 		text = g_strdup_printf ("%lu", year);
 	} else {
 		text = g_strdup (_("Unknown"));
@@ -975,8 +1097,12 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, album);
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			const char *entry_album;
+			
 			entry = (RhythmDBEntry *)tem->data;
-			if (strcmp (album, rb_refstring_get (entry->album)) == 0)
+			entry_album = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
+			
+			if (strcmp (album, entry_album) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
 					    RHYTHMDB_PROP_ALBUM, &val);
@@ -989,8 +1115,12 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, artist);
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			const char *entry_artist;
+			
 			entry = (RhythmDBEntry *)tem->data;
-			if (strcmp (artist, rb_refstring_get (entry->artist)) == 0)
+			entry_artist = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+			
+			if (strcmp (artist, entry_artist) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
 					    RHYTHMDB_PROP_ARTIST, &val);
@@ -1003,8 +1133,12 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, genre);
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			const char *entry_genre;
+
 			entry = (RhythmDBEntry *)tem->data;
-			if (strcmp (genre, rb_refstring_get (entry->genre)) == 0)
+			entry_genre = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
+			
+			if (strcmp (genre, entry_genre) == 0)
 				continue;
 			rhythmdb_entry_set (dialog->priv->db, entry,
 					    RHYTHMDB_PROP_GENRE, &val);
@@ -1049,8 +1183,12 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 		g_value_set_ulong (&val, discn);
 
 		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			ulong entry_disc_num;
+			
 			entry = (RhythmDBEntry *)tem->data;
-			if (discn != entry->discnum) {
+			entry_disc_num = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
+
+			if (discn != entry_disc_num) {
 				rhythmdb_entry_set (dialog->priv->db, entry,
 						    RHYTHMDB_PROP_DISC_NUMBER, &val);
 				changed = TRUE;
@@ -1067,27 +1205,38 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 static void
 rb_song_info_sync_entry_single (RBSongInfo *dialog)
 {
-	const char *title = gtk_entry_get_text (GTK_ENTRY (dialog->priv->title));
-	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
-	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
-	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
-	const char *tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
-	const char *discnum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
-	const char *year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
+	const char *title;
+	const char *genre;
+	const char *artist;
+	const char *album;	
+	const char *tracknum_str;
+	const char *discnum_str;
+	const char *year_str;
+	const char *entry_string;
 	char *endptr;
 	GType type;
-	gint tracknum;
-	gint discnum;
-	gint year;
+	gulong tracknum;
+	gulong discnum;
+	gulong year;
+	gulong entry_val;
 	GValue val = {0,};
 	gboolean changed = FALSE;
 	RhythmDBEntry *entry = dialog->priv->current_entry;
 
+	title = gtk_entry_get_text (GTK_ENTRY (dialog->priv->title));
+	genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
+	artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
+	album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
+	discnum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->disc_cur));
+	year_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->year));
+
 	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[PRE_METADATA_CHANGE], 0,
 		       entry);
 
 	tracknum = g_ascii_strtoull (tracknum_str, &endptr, 10);
-	if ((endptr != tracknum_str) && (tracknum != entry->tracknum)) {
+	entry_val = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER);
+	if ((endptr != tracknum_str) && (tracknum != entry_val)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_TRACK_NUMBER);
 		g_value_init (&val, type);
@@ -1098,7 +1247,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 	
 	discnum = g_ascii_strtoull (discnum_str, &endptr, 10);
-	if ((endptr != discnum_str) && (discnum != entry->discnum)) {
+	entry_val = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER);
+	if ((endptr != discnum_str) && (discnum != entry_val)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_DISC_NUMBER);
 		g_value_init (&val, type);
@@ -1109,13 +1259,18 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 	
 	year = g_ascii_strtoull (year_str, &endptr, 10);
+	entry_val = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_YEAR);
 	if ((endptr != year_str) && 
-	    (entry->date == NULL || year != g_date_get_year (entry->date))) {
+	    (year != entry_val || 
+	     (entry_val == 0 && year > 0))) {
 		GDate *date = NULL;
 	
 		if (year > 0) {
-			if (entry->date) {
-				date = g_date_new_julian (g_date_get_julian (entry->date));
+			if (entry_val > 0) {
+				gulong julian;
+
+				julian = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DATE);
+				date = g_date_new_julian (julian);
 				g_date_set_year (date, year);
 			} else {
 				date = g_date_new_dmy (1, G_DATE_JANUARY, year);
@@ -1135,7 +1290,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	}
 
 
-	if (strcmp (title, rb_refstring_get (entry->title))) {
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_TITLE);
+	if (strcmp (title, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_TITLE);
 		g_value_init (&val, type);
@@ -1146,8 +1302,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		changed = TRUE;
 	}
 
-
-	if (strcmp (album, rb_refstring_get (entry->album))) {
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ALBUM);
+	if (strcmp (album, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_ALBUM);
 		g_value_init (&val, type);
@@ -1158,7 +1314,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		changed = TRUE;
 	}
 
-	if (strcmp (artist, rb_refstring_get (entry->artist))) {
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_ARTIST);
+	if (strcmp (artist, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_ARTIST);
 		g_value_init (&val, type);
@@ -1169,8 +1326,8 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 		changed = TRUE;
 	}
 
-
-	if (strcmp (genre, rb_refstring_get (entry->genre))) {
+	entry_string = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_GENRE);
+	if (strcmp (genre, entry_string)) {
 		type = rhythmdb_get_property_type (dialog->priv->db,
 						   RHYTHMDB_PROP_GENRE);
 		g_value_init (&val, type);
@@ -1194,6 +1351,9 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 static void
 rb_song_info_sync_entries (RBSongInfo *dialog)
 {
+	if (!dialog->priv->editable)
+		return;
+
 	if (dialog->priv->current_entry)
 		rb_song_info_sync_entry_single (dialog);
 	else
